/* This file is part of libmt library
   Copyright (C) 2002  Multitran Project <libmt@multitran.ru>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

#ifndef __CABCSUBJ_H     /* prevent multiple includes */
#define __CABCSUBJ_H

#include "btree.h"
#include "trabuff.h"

#define CREATE_SUBJECT_DATABASE 1 // parameter to OpenAbcSubject

class AllTra : public BTree
{
public:
	AllTra (int langin = 0, int langout = 0);
	int	SeekEntry (TraBuff &TB);
	int	AppendEntry (TraBuff &TB);
	void	FormEntry (TraBuff &TB, Cbyte &buff);
	int	OpenAllTra (int langin, int langout, Cbyte &datapath = data.m_datapath);
	int	DeleteEntry (TraBuff &TB);
	const void	ReOpenDatabases (int justclose = 0);
};

class AbcSubject : public BTree
{
public:
	AbcSubject (int subjcode = 0, int lalgin = 0, int langout = 0);
	~AbcSubject();

	int	FormRecord (Cbyte orig, int subj, Cbyte &ret);
	int	OpenAbcSubject (int langin, int langout, Cbyte &datapath = data.m_datapath);
	long	GetTermNumber();
	int	AppendAbcRecord (TraBuff &TB);
	int	DeleteAbcRecord (TraBuff &TB);
	int	LocateAbcRecord (TraBuff &TB);
	int	GetAbcLine (Cbyte &buff);
//   int	FormSubjectPath (int langin, int langout, Cbyte &path1);
	int	FormRecord (TraBuff &TB, Cbyte &ret);
	void	Rebuild (int lang1, int lang2);
	int	LocateAbcRecord (Cbyte &buff);
	long	CountRecords();
	void	GoBottom();
	void	GoTop();
	int	GetSubject();
	int	Skip (int dir = 1);
	int	GoToLine (long line);
	static const void	ReOpenDatabases (int justclose = 0);
	static const void	UpdateEntry (TraBuff &newTB, TraBuff &oldTB);
	int	Bof();
	int	Eof();

	int	SetSubject (int code);
	int	m_DeleteBeforeAppending;
	
private:
	int	m_SubjectCode;
};

extern AbcSubject *GetAbcSubjectPointer (int langin, int langout, Cbyte &datapath = data.m_datapath);
extern AllTra *GetAllTraPointer (int langin, int langout, Cbyte &datapath = data.m_datapath);
#endif
