/* This file is part of libmt library
   Copyright (C) 2002  Multitran Project <libmt@multitran.ru>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

#ifndef __BTREE_H     /* prevent multiple includes */
#define __BTREE_H

#include "cbyte.h"

#define MAX_FIELD_SIZE 255

#define MAXABCSUBJECTS		2
#define MAXPHRASEINDEXES	5 // 5
#define MAXINDEXES			6 // 6
#define MAXPREFIXES			3 // 3
#define MAXSTEMS				4 // 4
#define MAXTYPEINS			4 // 4
#define MAXUSERTRA			4 // 3
#define MAXAUTORS				MAXLANGS
#define MAXTRANSCRIPTIONS	MAXLANGS
#define MAXENDINDEXES		MAXLANGS

#define APPEND_UNIQUE 1    // used by addrecord
#define APPEND_ALL    2

// for cycle operations
#define PROCESS_FINISHED       1
#define PROCESS_INTERRUPTED    -1

#define UPPERLEVEL_HEADER 'U'
#define LOWERLEVEL_HEADER 'L'

#define SYSIDENT 'M'
#define VERSIONMAX    15 // size of version portion
#define STANDARD_LEVEL_SIZE 4096 // 2048

#define BIG_LEVELSIZE 4096 // level size for alltra, dict.ert

struct DbHead  {
  __int16 DbIdentifier;          // 0
  __int16 EncodePage;            // 2
  long    AnchorOffset;          // 4
  __int16 Void4;                 // 6
  __int16 DbLanguage;            // 8
  long    MaxNumber;             // 12
  DWORD   CheckSum;              // 16
  DWORD   Void5;                 // 20
  __int16 DbType;                // 24
  __int16 Void6;                 // 26
  __int16 LevelSize;             // 28
  __int16 CodeTable;             // 30    Windows or DOS
  __int16 DbVersion;             // 32
  long    MaxRecords;            // 36  Number of records in the database
  long	  MaxInformationBlocks;
};

// for fileinfo functions
#define IDOFF         0    // where file identifier is written
#define ANCOFF        4    // where file offset to anchor level is written
#define LANGOFF       8    // where language identifier is written
#define MAXNUMBASEOFF 12   // where current maximum base number is written
#define CHECKOFF      16   // where file checksum is  written
#define DBTYPEOFF     24   // dbase type offset
#define VERSIONOFF    28   // version name offset
#define LANGOFF1      200  // language 1
#define LANGOFF2      202  // language 2
#define LEVELSIZEOFF  204  // offset to level size info

#define MAXLEVELS 10  /* Max number of database levels */

#define SEEK_FIRST_OF_SAME 1    // seek strategy
#define SEEK_LAST_OF_SAME  2
#define SEEK_CLOSEST_MATCH 4
#define ADD_FIRST_RECORD	8    /* append entry as first, used as OR with DATA.ACTION */

#define DB_ENCODE_PAGE     1   // database encoding constants
#define DB_ENCODE_ACTIVE   2
#define DB_ENCODE_PASSIVE  4

// used by IsSameRecord
#define	CHECK_FORWARD 1			// compare with next record
#define	CHECK_BACKWARD -1			// compare with previous record
#define	NO_MOVE_BACK  0			// don't move back if next record is the same
#define	MOVE_BACK  1				// move back if next record is the same
#define	CHECK_WHOLE_RECORD -1	// compare whole record's size

#define	MOVE_TO_TOP			1 // parameters for Get...Pointer functions
#define	CREATE_DATABASE	2
#define	CREATE_DATABASE_IF_NOT_EXISTS	1

class Cbyte;

// Basic Oubase b-tree class
class BTree
{
 public:
  Cbyte m_dbname;
  Cbyte	m_datapath;
  int m_langs[2];

  int	CompareRecordToNext ();
  int	m_optimized;
  int	GetAnchorLevelOccupiedSize();
  int	GetHandle();
  BTree (/*int levelsize = 0, int codepage = 0*/);
  int InitClass (int levelsize);
  ~BTree ();
  int InitDb (int levelsize);
  int	Check (char *actp, int actlen, char *passp, int passlen);
	
  int	AddRecord (int num, Cbyte &buff);
  int	AddRecord (Cbyte &buff, int num);
  int	AddRecord (BTree &BT);
  int	CreateFromDb (BTree *Bptr, byte *fname);
  int	GetFreeLevelSpace (void);
  void	GotoFirstRecord (void);
  int	SeekLevel (char *where, char *what, int size, int *resoffset);
  int	SeekLevel1(byte *where, char *what, int size, int *resoffset);
  int	Eof (void);
  int	Bof (void);
  int	OpenDb (char *fname = NULL, char *ext = NULL, int CreateIfNotExists = 0);
  int	OpenDb (Cbyte &fname, int CreateIfNotExists = 0);
  int	OpenDb (byte *fname, int CreateIfNotExists);
  int	OpenAnyDb (char *path, char *fname);
  void  SetWriteMode ();
  int	CloseDb (void);
  int	IsOpenDb (void) const;  // rets 1 if file is opened in the current workarea
  int	HardSeek (byte *word1, int size);
  void	SetError (int);
  int	GetError (void);
  int	ReadPage (int alwaysread = 0);
  int	AbsoluteSkip(void);
  long  CheckDb (void);
  char *CPointAct (void);
  char *CPointPass (void);
  inline char *PointAct (void); // const;
  char *PointPass (void); // const;
  inline char *PointFree (void) const;
  inline int	GetFreeOffset (void) const;  /* returns offset to curr. level's free space   */
  inline int	ActSize (void); // const;
  inline int	PasSize (void);
  long	GetRecordNumber (void) const;
  int	RestDbPos (long offset); // Gotorec + seek to ensure proper positioning
  int	GoToRecord (long offset);   // goto db position specified by OFFSET + seek
  int	GoToRecord1 (long offset);   // goto db position specified by OFFSET
  int	WritePage (void);
  int	IsFirstRecord (void) const;
  int	IsLastRecord (void);
  int	GetLevel (void);
  inline int	IsUpperLevel (void) const;
  inline int	IsLowerLevel (void) const;
  inline int	IsValidRecord (void); // const;
  int	DeleteRecord (void);
  int	Skip (int direct = 1);
  int	QuickSkip (void);
  inline int	IsLastLevel (void);
  int	EditPassive (char * passp, int passlen);
  int	EditPassive (Cbyte &buff);
  int	AddRecordConvertActive (char *actp, int actlen, char *passp, int passlen, int type = APPEND_ALL);
  int	AddRecordConvertPassive (char *actp, int actlen, char *passp, int passlen, int type = APPEND_ALL);
  int	AddRecord (char *actp, int actlen, char *passp, int passlen, int type = APPEND_ALL);
  int	AddLastRecord (char *actp, int actlen, char *passp, int passlen);
  int	SetAppendStrategy (int divplace, int deltavalue);
  int   IsSameRecord (int direct = CHECK_FORWARD, int goback = MOVE_BACK, int maxcompare = CHECK_WHOLE_RECORD);
  int	SeekConvertActive (char *actp, int actlen);
  int	Seek (byte *buff, int size, int strategy = SEEK_FIRST_OF_SAME);
  int	Seek (Cbyte &buff, int size) {return Seek (buff.b, size);};
  int	Seek (Cbyte &buff);
  int	Seek (long num);
  int	Seek (int num);
  int	AddRecord (Cbyte &buff, Cbyte &buff1);
  int	GoTop (void);
  int	GoBottom (void);
  int	CreateDb (char *name, char *ext = NULL, int levelsize = STANDARD_LEVEL_SIZE);
  int	CreateDb (Cbyte &buff);
  void	SetWriteAccess();
  int	ReadLevelSize (void);
  int	WriteLevelSize (int leveloff);
  int	ConvertToLevelSize (int levelsize);
  long	Rescue ();
  long	InvertRecords(); // reverse act/pass parts
  int	DumpContents (char *fname, int typetext = 0); // dump act &pass
  int	CheckLevel (int type = 0);
  int	DumpBinary();
  long	GetOffsetToNextLevel();
  long	CopyDb();
  int	SkipSpecial ();
  int	AppendFromDb (BTree *DbSource); // Append all new records from DbSource to current database
  long	Optimize (int levelsize = 0);
  long	Optimize1 (int levelsize, int pass);
  void	ReOpenDb (int justclose = 1);
  int	FormRec (Cbyte &buff1);
  static int RandomString (char *buff); // creates random string for cycle appending tests
  int	AddRecordSplitPassive (byte *act, int actsize, byte *pass, int passize, int delta = 1);
  int	AddPassiveSplit (char *toadd, int size);
  int	AddPassive (Cbyte &buff);
  int	SplitRecordPasSize();
  int	CollectSplitRecordPassive (Cbyte &buff);
  int	ReCreateDb();
  void	SetDbName (byte *name);
  int	DeleteDb();
  int	GoToRecord (long offset, int off);
  long	GetMaxRecords() const;
  int	GetLevelSize() const;
  int	GetDbLanguage() const;
  void	SetDbLanguage (int val);
  struct DbHead *BTree::GetDbHeaderPtr (void);
  void	AdjustMaxRecords (void);

  // Locking functions
  int	DoLock();
  int	DoUnLock();
  long	m_MaxLock;

  int	GoToLine (long line);
  long	GetFileSize();
  long	GetFileOffset();

  int	GetDbType (void) const;
  int	SetDbType (int);
  int	GetDbLang (void);
  long	GetMaxNumber (void) const;
  int	SetMaxNumber (long);
  int	AddPassive (char * toadd, int size);
  int	DBCheck (void);
  int	ReadManyBlocks (long termnum, char * whereput, int maxsize);
  int	DeleteManyBlocks (long termnum);
  int	WriteManyBlocksDirect (long termnum, char *whatput, int putsize);
  int	WriteManyBlocks (long termnum, char *whatput, int putsize);
  int	ConvertFileToVersion2_0 (char *argc);
  int	ConvertFilesToVersion2_0 (void);
  int	SkipBackward (void);
  int	SkipForward (void);
  void	SavePosition();
  void	RestorePosition();
  int	GetDbIdent ();
  int	SetLang  (int curhand, int lang);
  int	CopyStructureFromDb (BTree *BTreePtr);
  char *GetDbName();
  void	MakeEconomic (void); // use minimum disk space
  int	ReassignBuffer (int level, int size);
  int	ReadHeader();
  int	WriteHeader();
  void	ConvertStringFromDb (Cbyte &buff);
  void	ConvertStringToDb (Cbyte &buff);
  static const DWORD DiskFreeSpace (Cbyte &path); // IsNoun (int lgk);

  //	Cbyte m_CurrentDataPath; // = "" for standard paths

  int	m_maxlevels;  // max tree heigth
  int	m_alwayswrite; // always save changed level
  int	m_anchorsize;     // used space on anchor level
  long	m_CurRecNumber;  // stores current record number; it is reset to 0 after each Seek
  long	m_ValidCounter; // = 1 if m_currecno count from top of file; = 0 if m_currecno counts from last Seek position
  int	m_DeleteOnClose;
  Cbyte	m_SaveAct;
  int	m_SaveSize;

  // open parameters
  int	m_open_type;
  int	m_open_access;
  int	m_open_readwrite;
  int	m_UseGlobalAccessParameters; // passed to DbOpen, =1 - check data.m_doappend in AddRecord(), = 0 - do not check

  // level overflow parameters
  int	m_dividesize;
  int	m_delta;
  int	m_maxread;

  DWORD m_HeaderCheckSum;  // set during ReadHeader
  int	m_DoNotContractPage; // ReadPage reallocates page size only when the page extends but not when it contracts
  int	m_ErrorMax;
  int	m_eof;
  int	m_PageSize[2];     // current page size
  int	m_level;          // current level
  int	m_curhand;
  char *m_curbuff;
  long  m_FileOffset[MAXLEVELS];
  int	m_oklevel;                // level is ok (???)
  int	m_MaxWrittenPages; // counts calls to WritePage 
  long	m_PageCount; // to monitor progress in AbsoluteSkip
  int	m_AppendAction;
  struct DbHead m_DbHeader;  // database header information

 protected:
  int	m_waschange;   // there was change to information
  int	m_DbLang;                 // database language
  int	m_DbType;                 // database type

  // database access
  int   m_LevelOffset[MAXLEVELS];
  char *m_buff[2];

  long	GetAnchorOff (void) const;
  int	SetAnchorOff (int hand, long var);
  int	Ok (void);
  inline int	IsEndOfLevel (void);
  int	GoToLastRecord (void);
  int	RecordStart (int offset);
  int	DefineShortest (int beg, int end, char * delim);
  int	CanAdd (void);
  int	LevelSpace (void);
  int	AddToUpper (char *newact, int newactsize, long endoff);
  int	OverFlow (void);
  int	UpOverFlow (void);
  int	Append (char * actp, int actlen, char * passp, int passlen);
  int	SetFreeOffset (int value);
  int	DeleteRecordInMemory (void);
  inline int	PreviousRecord (void);
  inline int	NextRecord (void);
  inline int	NextLevel (void);
  inline int	PrevLevel (void);
  int	CheckDbPos (void);
  inline char *PointRecord (void); // const;
  inline int	GetRecordOffset (void); // const;
  int	RecordSize (void);
  int	CreateUpper (char *newact, int newactsize, long curpageoff, long lastpageoff);
  int	InitializeDb (int handle, int id); // used by CreateDb

  inline long	StringToFileOffset (char * string);
  char *FileOffsetToString (long l);

  char *m_delim;
  uint  delimsize;
  long	willwriteto;
  long	its2bytes;
  int	m_version;     // program version
  int	m_version1;    // program version
  long  m_slong;     // used by LongToChar and CharToLong functions
  int	m_FirstSeek;
  Cbyte m_pointact;   // pointer to ABC-converted active part
  Cbyte m_pointpass;  // pointer to ABC-converted passive part

  BTree *m_db;  // for temp usage
};

// check database
class CheckDb : public BTree
{
 public:
  CheckDb (char *fname);
  ~CheckDb();
  int TestSeek();

  BTree tmp;
};

// Types of database
#define DB_STEM  1
#define DB_INDEX 2
#define DB_DICT  3

// String encoding class
#define MAXMATRIX 256
class EnCode
{
 public:
  static const void EnCodeString (char *what, int size, DWORD offset);
  static const void DeCodeString (char *what, int size, DWORD offset);
 private:
  const static char m_CodeMatrix[MAXMATRIX];
};

class MessageDb : public BTree
{
 public:
  MessageDb();
  ~MessageDb();

  int	SetMessage (long code, char *mess);
  int	SetLang (int lang);
  int	FormString (char *buff);
  long	GetMessageNumber ();
  long	GetMaxMessageNumber ();
  byte *GetMessageText (long code, byte *defaults = "");

  Cbyte m_messname;
  Cbyte m_MessageDatabaseName;
  int	m_HeaderNumber;
};

class TipDb : public MessageDb
{
 public:
  TipDb (int lang);
  Cbyte GetNextTip ();
  Cbyte m_LastTip;
};

extern MessageDb G_Message;
#define LOGFILENAME "mt.log"
#define OURBASE_OK       1

class MtStatistics : public BTree
{
 public:
  MtStatistics();
  ~MtStatistics();
  int OpenStatistics();
  int AppendStatistics (Cbyte &buff);
};


#endif  // __BTREE_H
