/* This file is part of libmt library
   Copyright (C) 2002  Multitran Project <libmt@multitran.ru>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

// My string class
#ifndef __MYSTRING_H     /* prevent multiple includes */
#define __MYSTRING_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ohead.h"
#include "ofunc.h"
#define NO_DOS_CONVERT		0 // for Fgets
#define YES_DOS_CONVERT		1 // for Fgets

extern int IsOdd (int i);
extern int IsEven (int i);

class Cbyte {
 public:
  Cbyte (int max = 0);
  Cbyte (char *val); 
  Cbyte (Cbyte &val);
  Cbyte (const Cbyte &val);
  Cbyte (char *buff, int size);
  ~Cbyte();

  void	InsertAtStart (char *buff) {
    Cbyte t;
    t = buff;
    t += b;
    Strcpy (t.b);
  };
  int	Filled();
  void	ExtractFileNameAndExtension ();
  void	EraseBrackets (byte *start, byte *end, int clearblanks = 1);
  void Cbyte::EraseSymbols (char *buff) {
    for (int i = 0; b != NULL && b[i]; i ++)
      for (int j = 0; buff[j]; j ++)
	if (b[i] == buff[j])
	  b[i] = ' ';
    FindReplace ("  ", " ");
    AllTrim();
  };

  int Cbyte::IsRomanNumber ()
    {
      byte romans[] = "IVX";
      int match;
      for (int i = 0; b[i]; i ++) {
	match = 0;
	for (int j = 0; romans[j]; j ++) {
	  if (b[i] == romans[j])
	    match = 1;
	}
	if (!match)
	  return 0;
      }
      return 1;
    }

  void	AddBackSlash();
  void	FreeObject();
  void	EraseTrailingSymbols (byte *what);
  void	ReplaceSymbolInBrackets (byte insym, byte outsym);
  int	EraseStringInBrackets (byte *openbr, byte *closebr);
  int	GetStringInBrackets (byte *openbr, byte *closebr, Cbyte &out);
  void	ExtractPathAndFileName ();
  void	DeleteElements (Cbyte &elements, byte *delim = " ");
  void	DeleteSymbolsLowerThan32 ();
  void	DeleteEnds (Cbyte &ends);
  void	EraseBrackets (int type);
  void	EraseAllBrackets ();
  int	DeleteLinePart (int num, byte *delim);
  int	ContainsAnyOf (byte *str, byte *delim = ";")
    {
      Cbyte tmp;
      for (int i = 0; tmp.GetLinePart (str, i, delim); i++) {
	tmp.AllTrim();
	if (StrStr (tmp.b) != NULL)
	  return 1;
      }
      return 0;
    };
  int	IsAnyOf (byte *str, byte *delim = " ")
    {
      Cbyte tmp;
      for (int i = 0; tmp.GetLinePart (str, i, delim); i++) {
	if (tmp == b)
	  return 1;
      }
      return 0;
    };
  int	IsAnyOf (int off, byte *delim)
    {
      for (int i = 0; delim[i]; i ++)
	if (b[off] == delim[i])
	  return 1;
      return 0;
    };
  int	EndsWithAnyOf (byte *str, byte *delim = " ")
    {
      Cbyte tmp;
      for (int i = 0; tmp.GetLinePart (str, i, delim); i++) {
	if (HasEnd (tmp))
	  return 1;
      }
      return 0;
    };
  int	GetBufferSize() {return m_max;};
  void	EraseFinal (byte *val) {if (HasEnd(val)) Shorten(1); AllTrim();};
  int	CountCharacter (byte val);
  void	KillInBrackets();
  void	Reverse (byte *delim);
#ifdef WIN32
  void AddSlash () {if (!HasEnd("\\")) Strcat ("\\");};
#else
  void AddSlash () {if (!HasEnd("/")) Strcat ("/");};
#endif
  int	Atoi() {if (b == NULL) return 0; return (atoi (b));};
  int	Atol() {if (b == NULL) return 0; return (atol (b));};
  int	IsValid() {return (!m_max) ? 0 : 1;};
  int	IsOneOf (byte *buff);
  void StrRev () { 
#ifdef WIN32
    strrev (b);
#else 
    int i, ch, hlen, len;
    i = 0;
    len = strlen(b);
    hlen = (len)/2;
    while(i < hlen) {
      ch = b[i];
      b[i] = b[len-i-1];
      b[len-i-1] = ch;
      i++;
    }
#endif
  };
  void AppendByte (byte byteval) {
    Cbyte tmp(2);
    tmp.b[0] = byteval;
    tmp.b[1] = 0;
    Strcat (tmp.b);
  }
  byte *SetValue (byte *buff, int max);
  inline void	SetSize (int max) {
    if (max < m_max)
      b[max] = 0;
  }
  inline void operator = (LPCTSTR buff) {Strcpy (buff);};
  inline void operator = (char *buff)  {Strcpy (buff);};
  inline void operator = (Cbyte &cb) {Strcpy ((LPCTSTR)cb.b);};
  inline void operator += (Cbyte &cb) {Strcat (cb.b);};
  inline void operator += (byte *buff) {Strcat (buff);};
  inline void operator += (byte c) {
    Cbyte t;
    t = " ";
    t.b[0] = c;
    Strcat (t.b);
  };
  inline void AllTrim() {alltrim (b);};
  inline void RightTrim() {rtrim (b);};
  inline void LeftTrim() {ltrim (b);};
  inline void Shorten (int max) {shorten (b, max);};
  inline void IncrementLastLetter (void) {if (StrLen ())  b[StrLen () - 1] ++; };
  inline void DecrementLastLetter (void) {if (StrLen ())  b[StrLen () - 1] --; };
  inline void Clear (void ) {Strcpy(" "); b[0] = 0;};

  int HasEnd (Cbyte &str, int type = CHECK_ONE_FORM);
  inline int HasEnd (LPCTSTR str, int type = CHECK_ONE_FORM) {
    return hasend ((LPCTSTR)b, (LPCTSTR)str, type);
  }
  int HasStart (LPCTSTR str);
  inline int GetMaxLineParts (byte *delim = " ") {return (::GetMaxLineParts ((LPCTSTR)b, delim));};
  inline int IsEmpty () { if (!m_max)	return 1;else return (!b[0]);	};
  inline int StrLen () {
    if (!m_max)
      return 0;
    else
      return (strlen ((LPCTSTR)b));
  };
  inline int operator != (byte *val) {
    if (!m_max || val == NULL)
      return 0;
    else
      return strcmp ((LPCTSTR)b, (LPCTSTR)val);
  };
  inline int operator != (Cbyte &val) {
    if (!m_max || !val.m_max)
      return 1; // one of the strings is empty, return 'mismatch'
    else
      return strcmp ((LPCTSTR)b, (LPCTSTR)val.b);
  };
  inline int operator == (Cbyte &val) {
    if (!m_max || val == NULL || val.b == NULL)
      return 0;
    else
      return !strcmp ((LPCTSTR)b, (LPCTSTR)val.b);
  };
  inline int operator == (byte *val) {
    if (!m_max || val == NULL)
      return 0;
    else
      return (!strcmp ((LPCTSTR)b, (LPCTSTR)val));
  };

  int	IsEqualToAnyOf (byte *buff);
  int	GetLinePart (Cbyte &val, int i, byte *delim = " ") {return (GetLinePart ((LPCTSTR)val.b, i, delim));};
  int   GetLinePart (LPCTSTR val, int i, byte *delim = " ");
  byte  *Assign (int max, int always = 0);
  void  ExtractPath();
  void	ExtractExtension();
  void	ExtractFileName (int cutextension = 1);
  void	DeleteExtension ();
  void  Strcpy (LPCTSTR buff);
  void  Strcat (byte *buff);
  byte	*StrStr (byte *buff);
  byte	*StrStr (Cbyte &buff);
  byte	*StrChr (int c);
  void	Extend (int size) {if (size < 0) return; while (StrLen() < size) Strcat (" ");};
  int	Fputs (FILE *hand, byte *line = (byte *)"\n");
  byte	*Fgets (FILE *hand, int convert = 1);
  int	FindReplace (byte *what, byte *whatto, int max = 200);
  int	FindReplace (Cbyte &what, Cbyte &whatto);
  int	FindReplace (Cbyte &what, const Cbyte &whatto);
  int	FindReplace (byte *what, Cbyte &whatto);
  void	InsertMark (byte *mark = (byte *)"@", int pos = 0);
  void	InsertSymbol (int pos, byte *sym)
    {
      if (pos > StrLen())
	return;
      Cbyte tmp;
      tmp = b + pos;
      b[pos] = 0;
      Strcat (sym);
      Strcat (tmp.b);
    };

  byte *b;
  int	m_max;
  //	int	m_langs[2];
  int	m_ConvertCharacters;
  int	m_CutLineBreak;
};

struct MyDataStruct {
  int m_HelpLanguage;
  int m_ShowTranscription;
  int m_DeleteRegistrySection;
  int m_TranscriptionFontInstalledCounter;
  int m_TranscriptionFontInstalled;
  int m_AutoProcessClipboard;
  int m_AutomaticTransliteration;
  int m_TranscriptionFontAvailable;
  int m_ShowComments;
  int m_ShowInclusions;
  int m_dodelete;
  int m_doappend;
  int m_domove;
  int m_domodify;
  int m_demo;
  int m_station;
  int m_net;
  int m_ansi;
  int m_freenet;
  int m_codetable;
  int m_suggest;
  int m_checksubject;
  int m_SynchronizeViewers;
  int m_SingleViewer;
  int m_ViewerHeaderLanguage;
  int m_MultiDisplaySupport;
  int m_MaxCalls;
  int m_DemoWeeks;
  int m_DeleteAction;
  int m_InsertAction;
  int m_DoubleClickAction;
  int m_EnterAction;
  int m_TabAction;
  int m_TranslateFromClipboard;
  int m_WarnLargeClipboard;
  int m_ClipboardProcessSize;
  int m_WordMacroVersion;
  int m_DefaultDictionary;
  int m_UseMultitranDialogFont;
  int m_UseScriptFile;
  int m_SubjectLineBreak;
  int m_AutomaticTypeIn;
  int m_CheckSubjects;
  int m_MaintainAdditionalIndexes;
  int m_VersionNumber;
  int m_DefaultAppendSubjectCode;
  int m_TipOfTheDay;
  int m_CheckWordCase;
  int m_DisplayTypeinTranslations;
  int m_TipOfTheDayCount;
  int m_DumpNewStems;
  int m_ActivateOnCtrlInsIns;
  int m_SearchWhatIsTypedIn;
  int m_MinPhraseSize;
  Cbyte m_ExternalDictionaryPath;
  Cbyte	m_datapath;
  Cbyte	m_StemPath;
  Cbyte	m_localpath;
  Cbyte m_CreateDbPath;
  Cbyte m_logpath;
  int	m_SuggestLevel;
  int	m_PageDownAction;
  int	m_SuggestType;
  int	m_CreateDatabasesIfNotExist;
  int	m_langin;
  int	m_langs[2];
  int	m_AutoAppendAbbreviations;
  int	m_SortSubjectsByNumberOfTranslations;
  int	m_CopyToClipboardUmlautsAsEnglish;
  Cbyte	m_MultitranExeName;
};
extern MyDataStruct data;
extern void	ExchangeBuffers (Cbyte &one, Cbyte &two);
extern byte *GetMultitranRegistrySectionName();
extern BOOL CopyDirectory (Cbyte &lpszSrc, Cbyte &lpszDst, Cbyte &ExcludeList);
extern void RecursiveMkDir (Cbyte &buff);
extern byte *EnglishGoLower (Cbyte &name);
extern byte *EnglishGoUpper (Cbyte &name);
extern byte *GetErrorFile();

MyDataStruct* GetUserDataPointer();

#endif // __MYSTRING
