/* This file is part of libmt library
   Copyright (C) 2002  Multitran Project <libmt@multitran.ru>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

// btree-bases phrase class
#ifndef __CPHRASE_H     /* prevent multiple includes */
#define __CPHRASE_H

#include "btree.h"
#include "cword.h"
#include "getbit.h"
#include "index.h"
#include "tprocess.h"

#define PHRASE_CHECK		0
#define PHRASE_COLLECT	1
#define DO_CHECK_SUBEJCT 1
#define NO_CHECK_SUBEJCT 0
#define COLLECT_MAX_PHRASES 10000

#define JUST_CHECK_PHRASE_EXISTS		1
#define USE_ONLY_FIRST_FOUND_BLOCK	2

class CPhrase : public BTree
{
public:
   CPhrase (int langin = 0, int langout = 0);

	int	CheckPhrases (long wordnum);
	int	CloseDb (void);
	int	CollectSubjectsForPhrases (TranslationProcess &TP, Cbyte &buff, int type = -1, int param = 0);
	int	CollectSubjectsForPhrases (byte *wordnum, int maxvariants, Cbyte &buff);
	int	AppendPhrase (byte *wordnumstr, int max, int subj, long termnum);
	int	SeekWordNumber (long wordnum, int subject = -1);
	int	OpenPhrase (int langin = 0, int langout = 0, Cbyte CurrentDataPath = "");
   int	StartBuildPhraseList ();
   int	ContinueBuildPhraseList ();
   int	AppendPhrase (byte *wordnum, int maxwordnums, byte *terms, int maxterms);
	int	DeletePhrase (long wordnum, long termnum);
	int	DeletePhrasesForWordNumbers (byte *wordnum, int maxsize, long termnum);
   int	CollectPhrases (byte *wordnum, int maxvariants, long *whereput, int maxout, int type);  // collect all phrases
	int	CollectPhrases (CWord &WordPtr, long *m_termnums, int maxout, int checksubject);
//   int	FormDbName (Cbyte &fname1, int langin, int langout);
	int	ReplaceWordNumber (long wordnum1, long wordnum2);
	int	GetMaxWordNumbers ();
	int	GetWordNumber (int i = 0);
	int	CopyReferencedWordNumbers (OperateBit &bitptr, Cbyte &path);
	long	GetTermNumber (int pos);
	long	GetMaxTermNumbers ();
	int	AppendPhrase (byte *wordnums, int maxwordnums, long termnum);
	int	AppendPhrase (long wordnum, int subj, long termnum);
	void	Dump();
	int	GetSubject();
	static const void ReOpenDatabases (int justclose = 0);

   Index *m_Iptr; // current index
   long	m_count;  // counter
	long	m_TotalMax;
   Cbyte m_TempName;  // temp name for rebuilding index
	int	m_NeedAllPhrases;
	int	m_NewTermNumber; // for web collectphrase
	TranslationProcess *m_TPptr;
};

class WebPhrase : public BTree
{
public:
	WebPhrase (int lang1 = 0, int lang2 = 0);
	~WebPhrase();
	int	OpenWebPhrase (int lang1 = 0, int lang2 = 0);
	int	SeekElement (int snum, long wordnum);
	int	CheckElement (void);
	int	AppendTB (void);
	int	ReBuild();
	int	GetWordNumber ();
	int	GetSubject ();
	int	GetOriginal (Cbyte &buff);
	int	CollectData (CWord &cword, Cbyte &buff, int snum, Cbyte *out, int max);

	byte	m_act[6];
	Index *m_Iptr;
	long	m_cnt;
	FILE *m_hand;
};

extern CPhrase *GetPhrasePointer (int langin, int langout, Cbyte &CurrentDataPath = data.m_datapath);
#define PHRASES_DB_NAME "phrases"

#endif
