/* This file is part of libmt library
   Copyright (C) 2002  Multitran Project <libmt@multitran.ru>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

// New mwp class functions
#ifndef __CWORD_H     /* prevent multiple includes */
#define __CWORD_H

#ifndef __OHEAD_H     /* prevent multiple includes */
#include "ohead.h"
#include "ofunc.h"
#endif
#include "cbyte.h"
#include "tmorf.h"

#define WORD_ANY       1
#define WORD_VALID     2

#define MLP struct mylinfo *
#define ML  struct mylinfo

struct  mywords {
   int	start;
   int	end;
   byte	type;
   byte	nextdelim;
   int	linfostart;
   int	linfosize;
//   int	maxphrase;
   int	menuelem;
   byte	ignore;       // ignore 'the', 'a'
   struct      mywords *prev;
   struct      mywords *next;
   MLP	mlp;
   int	wordcase;   // case in text
   byte	inclusion;
   long	usetermnum;
   int	delimiter;
   int	done;  // =1 to indicate that this word was processed by LemmString
	int	startpos; // used in cprocess.cpp
	int	MoreTranslationsToShow;
	int	m_lang;
   };

#define MWP struct mywords *
#define MW  struct mywords
#define MAXDELIMITERS   5  // max number of delimiters like '.', ',', etc.
#define MAXDELIMITERNUMBER   7  // max word number of delimiters like '.', ',', etc.
#define CHECK_FIRST_WORD 1 // used in IsSpeechPart
#define MAX_DELIMITER_WORD_NUMBER 7 // numbers of .,/ etc.

class CWord {
public:
   CWord (Cbyte &startbuff);
   CWord ();
   ~CWord ();

	void	FetchAllWords (Cbyte &buff);
	void	SetMatchingUpperCaseToTop ();
	void	FreeWrongCase (int type);
	void	FreeNotMainForms (void);
	void	FreeElements (MWP mwp);
	int	SortElementsBySpeechParts(MWP mwp = NULL);
	void	SortElementsByNumberOfTranslations(MWP mwp = NULL);
	void	SortElementsBySubjectOrder (MWP mwp);
   int	DumpData (byte *fname);
   int	SetBuffer (Cbyte &startbuff);
   int	GetMaxWords (int type = WORD_ANY);
   int	DeleteUnwantedWords (void);
   int	FreeWord (int num) {return (FreeWord (PointWord (num)));}
   int	FreeWord (MWP mwp);
   int	FreeElement (int wnum, int elnum);
   int	FreeElement (MWP mwp, int num);
   int	GetMaxElements (int wnum, int type = WORD_VALID) {return (GetMaxElements (PointWord (wnum, type)));}
   int	GetMaxElements (MWP mwp);
   int	RealToValidWord (int neednum);
   int	FreeElementExcept (MWP mwp, int num);
   int	ExchangeElements (MWP mwp, int start, int end);
   int	FreeAllWords (void);
   int	FetchWord (int num, Cbyte &buff, int type = WORD_VALID) {return (FetchWord (PointWord (num, type), buff));}
   int	FetchWord (MWP mwp, Cbyte &buff);
   int	GetMaxUnknownWords();
   int	ParseWords (int max = -1);
	int	ParseWordsAndDigits ();
   int	ExchangeElements (int word, int start, int end);
   int	IsSpeechPart (int word, int spart, int type = 0);
   int	SetSpeechPart (int word, int spart);
   int	RecommendSpeechPart (int word, int spart);
   int	GetNextNumberString ();
   int	GetFirstNumberString (int start, int end);
   int	FillElements();
   int	NextVariant();
   int	SelectLikelyElement (int num);
   int	DoLemm (MWP mwp);
   MWP	PointUnknownWord (int num = 0);
   MWP	PointWord (int num, int type = WORD_ANY);
   MWP	QuickInsertLastWord (MWP prevmwp, MWP cmwp);
   MWP	InsertLastWord (MWP cmwp = NULL);
   MWP	AssignWord (int num, MWP cmwp = NULL);
   MLP	FindSpeechPart (int nword, int needpart);
   MLP	AssignElement (int num, MLP cmlp = NULL) {return (AssignElement(PointWord (num), cmlp));}
   MLP	AssignElement (MWP mwp, MLP cmlp = NULL);
   MLP	PointElement (int wnum, int elnum, int type = WORD_ANY);
   MLP	PointElement (MWP mwp, int num);
	static const int	GetDelimiterNumber (byte c);
	int	ValidToRealWord (int neednum);
	void	AppendNotMainFormToStem ();
	void	AppendNotMainCaseToStem ();

	int	m_MaxIgnore;
   byte	m_variants[NEW_MAX_PHRASE_SIZE];
   byte	m_formstr[NEW_MAX_PHRASE_SIZE*3];
   int	m_formsize;
   int	m_start;
   int	m_end;
	Cbyte	m_buff;
	Cbyte m_CopyBuff;
   int	m_lang;

   static byte m_delimiters[];
   static long m_delimnums[MAXDELIMITERS];

private:
   MWP  m_startmwp;
   int  m_max;
};

extern int IsDelimiter (long wordnum);
#endif
