/* This file is part of libmt library
   Copyright (C) 2002  Multitran Project <libmt@multitran.ru>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

// Class works with paired databases: name/code and code/name indexes
#ifndef __CDBPAIR_H     /* prevent multiple includes */
#define __CDBPAIR_H

#include "btree.h"
#include "noclass.h"

#define NAME_EXISTS -1
#define CODE_EXISTS -2
#define UNIQUE_CODE 1
#define UNIQUE_NAME 2
#define OPEN_CODE 1
#define OPEN_NAME 2
#define FORCE_CLOSE 1

#define SUBJECT_NAME       "SUB"   // subjects
#define SHORT_SUBJECT_NAME "SSUB"  // short subjects
#define SUBJECT_TYPE_NAME  "NAME"  // to form 'subname' and 'ssubname' names
#define SUBJECT_TYPE_CODE  "CODE"  // to form 'subcode' and 'ssubcode' names

class DbPair {
public:
   ~DbPair ();
   DbPair (Cbyte fname = (Cbyte)"", Cbyte ext = (Cbyte)"", int type = OPEN_CODE | OPEN_NAME);
   int CloseDatabases (int type = 0);
   int OpenDatabases ();
   int CreateDatabases (Cbyte &fname, Cbyte &ext);
   int GetCode (Cbyte &name);
   int GetNextCode ();
   byte *GetName (int code);
   byte *GetNextName ();
   int DeleteName (Cbyte &name);
   int DeleteCode (int code);
   int AddNameAndCode (Cbyte &name, int code, int type = 0 /*UNIQUE_NAME | UNIQUE_CODE*/);
   int GetMaxCode ();
   int GetCodeFromName ();
   int GetCodeFromCode ();
   int CollectNames (int code, Cbyte &where);
   int ReplaceNameAndCode (Cbyte &name, int code, int type = 0);
   int ReadNamesToMemory (int type = 0);
   byte *GetNameFromMemory (int code);
   void ClearNamesInMemory();

   BTree m_DbName;
   BTree m_DbCode;
   Cbyte m_fname;
   Cbyte m_ext;
   int m_opentype;
   int m_alwaysclose;
   int m_maxcode;
   byte **m_names;  // to store names in memory for quicker access
   int m_namesinmemory;
   };
#endif
