/* This file is part of libmt library
   Copyright (C) 2002  Multitran Project <libmt@multitran.ru>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

// Translation index class

#ifndef __CINDEX_H     /* prevent multiple includes */
#define __CINDEX_H

#include "btree.h"
#include "getbit.h"
#include "trabuff.h"
#include "cword.h"
#include "cbyte.h"

#define	RECORD_INCLUSION				1
#define	RECORD_MATCH					2
#define	SEEK_CHECK_SUBJECTS			1
#define	SEEK_NO_CHECK_SUBJECTS		0
#define	OFFSET_TO_FIRST_TERMNUMBER 2

#define	MAX_TERMNUMBERS_IN_RECORD MAX_FIELD_SIZE/3 - 1 // don't forget 2 bytes for subject number - that's why - 1 !

class Index : public BTree {
public:
	Index();
	~Index();

	void	Export (Cbyte &fname);
	int	FindNextMatchingRecord();
	void	RecalculateInfoBlocks ();
//	Cbyte GetExtension (int langin, int langout);
	int	GetSubject();

	long	SeekWordNumbers (byte *wnums, int size, int checksubjects, int &inclusion, int &exact);
	long	SeekWordNumbers1 (byte *wnums, int size, int checksubjects, int &inclusion, int &exact, long &toff, long &inclusion_off);
	int	AppendNewIndexRecord (byte *buff, int size, int snum, long termnum);
	int	CompareWordNumbers (byte *wnums, int size);
	int	SeekRecordForSubject (byte *nums, int numsize, TraBuff &TB);

	const static byte *m_IndexNames[4];
	int	CloseDb (void);
	int	OpenIndex (int langin, int langout, Cbyte odatapath = (Cbyte)"");
	int	SetTermNumber (int i, long termnum);
	long	GetWordNumber (int i);
	inline	byte *PointWordNumber (int i) {
		return (PointAct() + i*3 + OFFSET_TO_FIRST_TERMNUMBER);
		}
	int		GetMaxTermNumbers (void);
	int		SeekWordNumber (long l, int checkinclusion = 0);
	int		GetMaxWordNumbers (void);
	int		DeleteTermPosition (int elem);
	int		GetTermNumberPosition (long termnum);
	int		IsValidWordNumber (int i);
	int		IsValidTermNumber (int i);
	inline	long		GetTermNumber (int i) {
		if (!IsValidTermNumber(i)) return (ERR_NO_SUCH_TERM);
		return (threetolong (PointPass() + i*3 + OFFSET_TO_FIRST_TERMNUMBER));
		}
	inline	byte		*PointTermNumber (int i) {
		return (PointPass() + i*3 + OFFSET_TO_FIRST_TERMNUMBER);
		}
	int		AppendTermNumber (byte *nums, int size, TraBuff &TB, int needpos = -1);
	int		DetermineAppendPosition (TraBuff &TB);
	int		DeleteTermNumber (byte *nums, int size, TraBuff &TB);
	int		CollectAccessibleTermNumbers();
	int		CollectAccessibleWordNumbers();
	int		DeleteTermNumber (long foffset, TraBuff &TB);
	int		ReplaceWordNumber (long iwordnum, long owordnum, Cbyte &needword, Cbyte &ignoreword);
	int		IsActiveSubject();
	int		AppendIndexBlock (byte *nums, int numsize, byte *pass, int passize);
	static const void		ReOpenDatabases (int justclose = 0);

	int		m_InactiveSubject;
	OperateBit m_BitArray;
};

//extern Index Indexes[MAXINDEXES];
extern Index *GetIndexPointer (int langin, int langout, Cbyte &datapath = data.m_datapath);

#endif   // __CINDEX
