/* This file is part of libmt library
   Copyright (C) 2002  Multitran Project <libmt@multitran.ru>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

// Language class

#ifndef __LANG_H     /* prevent multiple includes */
#define __LANG_H
// Language functions

#include "btree.h"
#include "cword.h"
//#include "cbyte.h"

extern void EraseNonletterSymbols (Cbyte &buff, Cbyte ignore = (Cbyte)"");
extern void ConvertToRussian (Cbyte &buff, Cbyte &out);
extern void ConvertToEnglish (Cbyte &buff, Cbyte &out);

extern int GetWorkingLanguagesFromText (Cbyte &buff);
extern int GetPrepositionCase (int lgk);
extern int GetSpeechPart (int lgk);
extern void GetSpeechPartName (int lgk, Cbyte &buff);
extern int IsNoun (int lgk);
extern int GetGender (int lgk);
extern int IsAnimated (int lgk);
extern int IsAdjective (int lgk);
extern int IsVerb (int lgk);
extern int IsPrich (int lgk);
extern int GetLgkGender (int lgk);
extern int IsLgkPerfect (int lgk);

extern int GetWordFormDescription (int lang, int lgk, int formnum, Cbyte &buff);
extern void GoLower (Cbyte &buff);
extern void EraseNonLetterSymbols (Cbyte &buff);
extern int ExtractSetLanguages (FILE *hand, int &lang1, int &lang2);
extern int SearchLine (Cbyte &where, Cbyte &what, byte *delim = " ", int lowercase = 1);

// Language codes
#define MYENGLISH		1
#define MYRUSSIAN		2
#define MYGERMAN		3
#define MYFRENCH		4
#define MAXLANGS		5

// Synthesis behavior
#define SYNTHESIZE_FIRST_FORM 1
#define SYNTHESIZE_ALL_FORMS  0

#ifdef WIN32
#define SIK_CLEAR_END_OF_CLASS (unsigned __int16) (0xFFFF >> 1)
#define SIK_END_OF_CLASS (unsigned __int16) ((1 << 15))
#define SIK_PRICH_START  (unsigned __int16) ((1 << 14) | 1)
#define SIK_NEXT_CLASS   (unsigned __int16) ((1 << 14) | 2)
#define SIK_NO_END       (unsigned __int16) ((1 << 14) | 3)
#define SIK_SAME_END     (unsigned __int16) ((1 << 14) | 4)
#define SIK_GEBORREN     (unsigned __int16) ((1 << 14) | 5)
#define SIK_MAX_END_NUMBER (unsigned __int16) (1 << 14)
#else
#define SIK_CLEAR_END_OF_CLASS (uint16_t) (0xFFFF >> 1)
#define SIK_END_OF_CLASS (uint16_t) ((1 << 15))
#define SIK_PRICH_START  (uint16_t) ((1 << 14) | 1)
#define SIK_NEXT_CLASS   (uint16_t) ((1 << 14) | 2)
#define SIK_NO_END       (uint16_t) ((1 << 14) | 3)
#define SIK_SAME_END     (uint16_t) ((1 << 14) | 4)
#define SIK_GEBORREN     (uint16_t) ((1 << 14) | 5)
#define SIK_MAX_END_NUMBER (uint16_t) (1 << 14)
#endif
#define COMMA_WORDNUMBER 2   // word number of comma, see CWORD.CPP

class EndNumbers : public BTree {
public:
   EndNumbers (int langnum = 0);
   byte *GetEnd (int num, Cbyte &end);
   int AppendEndNumber (int endnum, Cbyte &end);
   int OpenEndNumbers (int langnum);
   int m_lang;
};

extern EndNumbers *GetEndNumbersPointer (int langin);
#define MAXENDNUMBERS MAXLANGS

class SikTable {
public:
   SikTable (int langnum = 0);
   int OpenTable (int langnum);
   ~SikTable();
   int CloseSikTable();
   unsigned int *PointSikLine (int line);
   int GetMaxEnds (int sik);

   int m_lang;
   unsigned short int *m_ends;
   long m_TableSize;
	int m_MaxEnds;
};

class StressTable {
public:
   StressTable (int langnum = 0);
   int OpenStressTable (int langnum);
	void	WriteChanges();
	void	Dump();
   ~StressTable();

   void	CloseStressTable();
	long	*GetStressLinePointer (int line);
   int	GetStressLineSize (int line);
	int	GetStressLine (long *receive, int line, int maxsize);
	int	FindMatchingStressLine (long *stressbuff, int buffsize);
	int	CreateStressLine (long *stressbuff, int buffsize, Cbyte &wword);
	void	DumpLine (int line, Cbyte &wword);

   int	m_lang;
	int	m_WasChange;
   long	*m_StressBuffer;
   long	m_TableSize;
	long	m_MaxStressLines;
};


// Cases of words
#define	CASE_ALL_LOWER		1
#define	CASE_ALL_UPPER		2
#define	CASE_FIRST_UPPER	4
#define	CASE_MIXED			8

#define DEFOLD 1
struct LanguageData {
#ifdef DEFOLD
   byte *siktable;
   byte *suffix;
   byte *suffixsik;
#endif
   byte *abcd;
   byte *vowel;
#ifdef DEFOLD
   byte *consonant;
   byte *vowelin;
   byte *vowelout;
#endif
   byte *hissing;
#ifdef DEFOLD
   byte *prefix;
   byte *ends;
   byte *stop;
   byte *delimiter;
   int  *sikoffsets;
#endif
   byte *partends;
   byte *parts;
#ifdef DEFOLD
   byte *nounmask;
   byte *nounpad;
   byte *adjmask;
   byte *adjpad;
   byte *aaa7;
   byte *aaa8;
   byte *aaa9;
   byte *aaa10;
   int  excluhand;
   int  maxsiks;
#endif
   int  abcdsize;
   };

#define CHECK_FIRST_LETTER 1 // used by DetermineLanguage
#define SEEK_WORD		1
#define APPEND_WORD	2
// mlp->m_prich_type
#define PRICH_DELAVSHII 1
#define PRICH_DELAUSHII 2
#define PRICH_DELANNY	3
#define PRICH_DELAEMY	4
#define LANGUAGE_DIGIT  100

// for ConvertToLowercaseIfPossible (Cbyte &tmp, int type)
#define CONVERT_IF_IS_ONLY_LOWERCASE	1
#define CONVERT_IF_COULD_BE_LOWERCASE	2
#define CONVERT_ONLY_FIRST_UPPERCASE	4

class Lang
{
public:
   Lang (int num = 0);
   ~Lang ();

	void	GenerateAbbreviation (Cbyte &orig, Cbyte &abbrev);
	int	ConvertToLowercaseIfPossible (Cbyte &tmp, int type);
	void	DeleteAlternativeForms (Cbyte &buff);
	void	ConvertToProperCase (Cbyte &buff);
	int	FgetsLineWithCarry (FILE *hand, Cbyte &buff);
   static const void	GetSikFlagsDescription (DWORDLONG flags, Cbyte &buff);
	int	ReplaceEToYoInBuffer (Cbyte &orig, int domark = 0);
	int	GetMaxSik();
	int	SynthesizeWordForm (Cbyte buff, MLP mlp, int maskform, Cbyte *obuff);
	int	ConvertUpperCaseToBits (Cbyte &buff, Cbyte &bits);
	int	ConvertBitsToUpperCase (Cbyte &buff, byte *bits, int size);
	void  CreateBitTable();
	int   GetLgkDescription (int lgk, Cbyte &buff);
   int   IsSameLgk (int lgk1, int lgk2);
   int   IsVowel (unsigned char let);
   int   GetVowelMax (Cbyte &wword);
   int   GetCurLang (void);
   int   CloseLang (void);
   int   OpenLang (int lang);
   int   IsOpenLang (void);
   int   ReadSetup (int);
   int   CloseBase (int);
   int   OpenBase (int);
   int   IsLgkTransitive (int lgk);
   int   GetLgkNumber (int lgk);
   int   LikeAdject (int lgk);
   int   Bald (int lgk);   /* 1 if verb cannot be used with ,  - Russian only ! */
   int   Hairy (int lgk);    /* 1 if verb can be used with ,  - Russian only ! */
   int   ConvertSikFile (void);
   int   SetWordCase (Cbyte &buff, Cbyte &origword);
	static const int SummCase (int ourcase);
   int   HasNoLetterSymbol (Cbyte &wword); /* checks for non-letter symbols */
   int   GetPrichSuff (Cbyte &suffstring, int lgk, int prichnum, int varnum, Cbyte &where);
   int   GetRealForm();
   int   GetLastLgk();
   int   IsSolidWordform (int lgk, int formnum);
	int   GetEndString (int sik, Cbyte &buff1);
   int   GetMaskSpeechPartSik (int lgk);
   int   GetMaskPtr (int lgk, byte **ptr);
   int   GetMaskPtr1 (int lgk, byte **ptr);
   int   GetMaxsPartForms (int spart);
   int   ClearGrid (byte *buff);
   int   CheckWordForm (int lgk, int formnum);
   int   FillEnds (int sik, int lgk, Cbyte &buff1, int type = 0);
   int   IsMainForm (MLP mlp, int nonstrinct = 0);
	int   IsSolidForm (int lgk, int formnumber);
	void   EraseNonLettersForThisLanguage (Cbyte &wword, int clearblanks = 1);
	int   MakeInitialForm (byte *wword, int lgk);
   int   TestLetter (int symbol);
   int   IsUpperLetter (int symbol);
   int   IsLowerLetter (int symbol);
   int   GoLower (Cbyte &word1);
   int   GoUpper (Cbyte &word1);
   int   LowerLetter (int let);
   int   UpperLetter (int let);
   int   BrushString (byte *buff);      /* delete all non-letter symbols */
   int   TestCase (Cbyte &buff);
   int   ForceCase (Cbyte &buff, MLP mlp);
   int   ForceCase (Cbyte &buff, int needcase);
   int   GetSikLine (int sik, Cbyte &buff);
   int   SynthesizeWordForms (MLP mlp, Cbyte &wform, Cbyte &out, int type = SYNTHESIZE_ALL_FORMS);
	int	SynthesizeParticiples (MLP mlp, Cbyte &wform, Cbyte &out, int type = SYNTHESIZE_FIRST_FORM);
	int	SynthesizeParticiple (MLP mlp, Cbyte &wform, Cbyte &out, int number, int firstform = SYNTHESIZE_ALL_FORMS);
	int   SynthesizeAllWordForms (MLP mlp, Cbyte &wform, Cbyte &out, int type = SYNTHESIZE_ALL_FORMS);
   int   ExistsParticiple (int lgk, int type);
   int	AddParts (Cbyte &form, int lgk);
   int	GetUppercasePercentage (Cbyte &buff);
	int	GetAbcPercentage (Cbyte &buff, int max = -1);
	int	GetAbcPercentage2(Cbyte &buff);
	int	GetMaxLetters(void);
	int	GetAbcPercentageString (Cbyte &buff, int prec, int max);
	int	CountWords (Cbyte &buff);
	int	GetExceptionForms (MLP mlp, Cbyte &stem, Cbyte &receive, int allforms = 0);
	int	IsException (int sik);
	int	CountLetters (Cbyte &buff);
   byte *FetchSuff (int num);
   DWORD CheckEndNumber (int sik, int lgk, int endnum, int *m_lgk, int *m_prich, int *m_OkFormCount, DWORD *m_DuplicateForm, int geborren = 0/*for German split prefixes*/);
	byte *GetWordFormName (int lgk, int formnum);
	int	GetMaxFormsForPartOfSpeech (int spart);

	static const int	CheckCaseCompatibility (int basecase, int wordcase, int type);
	static const void	 StripControlSymbols (Cbyte &buff); 
	static const void	EraseNoLetterSymbolsForAllLanguages (Cbyte &buff);
	static const void	 EraseAllNonLetterSymbols (Cbyte &buff, int alltrim = 0);
   static const int DetermineLanguage (Cbyte &buff, int type = 0, int lang1 = 0, int lang2 = 0);
   static const int DetermineLanguage (byte ch);
	static const int DetermineLanguageOfFirstValidLetter (Cbyte &buff);
	static const int DetermineLanguageOfLastValidLetter (Cbyte &buff);
   static const int IsSameForm (Cbyte &tmp1, Cbyte &tmp2, int lang = 0);
	static const int GetLanguageNumber (Cbyte &buff);

   static int   MakeEnglishPlural (Cbyte &buff);
   static const byte ConvertToLanguage (byte buff, int lang = 0);
	static const void ConvertToLanguage (Cbyte &buff, int lang);
	const int	IsUniqueLetter (byte letter);
   static const char *GetShortSpeechPartName (int spart);
	static const void	ConvertToOtherLanguage (Cbyte &buff);
	static const int TransliterateEnglishToRusian (Cbyte &buff);

	int CheckWordForm (MLP mlp, DWORD needform);
	FILE	*OpenSikFile ();
	int	GetNextSikLine (Cbyte &buff);
	void	CloseSikFile();

   SikTable		m_SikTable; // not BTree !!!
   //EndNumbers *m_EndNumbersPtr;

	Cbyte m_abcd;
	Cbyte m_vowel;
	Cbyte m_hissing;
	Cbyte m_partends;
	Cbyte m_parts;
	int	m_abcdsize;
	int	m_definedlang;
	int	m_LanguageFilesNotPresent;

private:
   int m_lang;
   int m_DefaultLanguage;
   const static byte *m_ShortLanguageNames[MAXLANGS];
   const static byte *m_LanguageNames[MAXLANGS];
   const static byte m_engl[];
   const static byte m_russ[];
	FILE	*m_SikFileHandle;
};

class CbyteLang : public Cbyte
{
public:
	int	GetNumericParameter (byte *paramname);
	int	GetNumericParameter (Cbyte &paramname);
	Cbyte	GetStringParameter (Cbyte &paramname, Cbyte delim, int carryreplacements);
	Cbyte	GetStringParameter (byte *paramname, byte *delim = "=", int carryreplacements = 1);
   inline void operator = (Cbyte &cb) {Cbyte::Strcpy ((LPCTSTR)cb.b);};
   inline void operator = (byte *cb) {Cbyte::Strcpy ((LPCTSTR)cb);};
	Cbyte CbyteLang::GetUserNameWithEmailReference ();
};

extern Lang Langs[MAXLANGS];

/********* Parts of speech codes *************************/
#define         P_VERB                  128
#define         P_NOUN                  64
#define         P_ADJECTIVE             32
#define         P_PRICH                 512
#define         P_PRONOUN               16
#define         P_ABBREVIATION          1
#define         P_ADVERB                2
#define         P_INTERJECTION          3
#define         P_CONJUNCTION           4
#define         P_ARTICLE               5
#define         P_NUMERAL               6
#define         P_ORD_NUM               7
#define         P_PREPOSITION           8
#define         P_FORM			          9
#define         P_PARTICLE              10
#define         P_PREDICATIVE           11

#define         P_SPECIAL_PART_OF_SPEECH -1

#define				P_YA	0
#define				P_TY	1
#define				P_MY	2
#define				P_VY	3
#define				P_ON	4
#define				P_ONA 5
#define				P_ONO 6
#define				P_ONI 7

#define         N_MASCULINE             8
#define         N_FEMININE              16
#define         N_NEUTER                512 // 24
#define         P_EUPHEMISM             1024
#define         A_SUBSTANTIVE_ADJECTIVE 4
//#define         N_NEW_NEUTER            512

#define         N_ANIMATED              4
#define         N_CLEARANIMATED         255 - N_ANIMATED
#define         N_ORDINARY              0
#define         N_CLEARNUMBER           255 - 3
#define         N_SINGULAR              1
#define         N_PLURAL                2
#define         N_UNCHANGEABLE          32
#define         N_PRONOUN		          256

#define         V_SYA                   1
#define         V_GEBORREN              256
#define         V_UNDEFTRANS            0
#define         V_TRANSITIVE            32
#define         V_INTRANSITIVE          64
#define         V_TRANS_INTRANS         3

#define         V_UNDEFPERFECT          0
#define         V_PERFECT               8
#define         V_IMPERFECT             16
#define         V_PARTICIPLE3           2  // participle '' exists
#define         V_PARTICIPLE4           4  // participle '' exists
#define         P_PARTICIPLE1           1
#define         P_PARTICIPLE2           2
#define         P_PARTICIPLE3           3
#define         P_PARTICIPLE4           4

#define         A_SHORT                 16
#define         A_COMPARATIVE           8
#define         A_PRONOUN		          2 // pronoun adjective

// New forms bits
#define         F_NOMINATIVE            ((DWORDLONG)1 << 0)
#define         F_GENITIVE              ((DWORDLONG)1 << 1)
#define         F_DATIVE                ((DWORDLONG)1 << 2)
#define         F_ACCUSATIVE            ((DWORDLONG)1 << 3)
#define         F_INSTRUMENTAL          ((DWORDLONG)1 << 4)
#define         F_PREPOSITIONAL         ((DWORDLONG)1 << 5)
#define         F_MASCULINE             ((DWORDLONG)1 << 6)
#define         F_FEMININE              ((DWORDLONG)1 << 7)
#define         F_NEUTER                ((DWORDLONG)1 << 8)
#define         F_SINGULAR              ((DWORDLONG)1 << 9)
#define         F_PLURAL                ((DWORDLONG)1 << 10)
#define         F_SHORT                 ((DWORDLONG)1 << 11)
#define         F_COMPARATIVE           ((DWORDLONG)1 << 12)
#define         F_COMPARATIVE1          ((DWORDLONG)1 << 13)  // for English - big-bigger-biggest
#define         F_LITSO1                ((DWORDLONG)1 << 14)
#define         F_LITSO2                ((DWORDLONG)1 << 15)
#define         F_LITSO3                ((DWORDLONG)1 << 16)
#define         F_DEEPR                 ((DWORDLONG)1 << 17)
#define         F_INFINITIVE            ((DWORDLONG)1 << 18)
#define         F_IMPERATIVE            ((DWORDLONG)1 << 19)
#define         F_PAST                  ((DWORDLONG)1 << 20)
#define         F_PRESENT               ((DWORDLONG)1 << 21)
#define         F_FUTURE                ((DWORDLONG)1 << 22)
#define         F_ANIMATED              ((DWORDLONG)1 << 23)
#define         F_INANIMATED            ((DWORDLONG)1 << 24)
#define         F_PRICH                 ((DWORDLONG)1 << 25)
#define         F_NOUN                  ((DWORDLONG)1 << 26)
#define         F_VERB                  ((DWORDLONG)1 << 27)
#define         F_ADJECTIVE             ((DWORDLONG)1 << 28)
#define         F_ADVERB                ((DWORDLONG)1 << 29)
#define			 F_GERUND                ((DWORDLONG)1 << 30)
#define			 F_INDEFINITE            ((DWORDLONG)1 << 31)
#define			 F_SYA		             ((DWORDLONG)1 << 32)
#define         F_PRICH1                ((DWORDLONG)1 << 33)
#define         F_PRICH2                ((DWORDLONG)1 << 34)
#define         F_PRICH3                ((DWORDLONG)1 << 35)
#define         F_PRICH4                ((DWORDLONG)1 << 36)
#define         F_ADDITIONAL_FORM       ((DWORDLONG)1 << 37)
#define         F_PARTICIPLE		       ((DWORDLONG)1 << 38)
#define         F_PREPOSITION		       ((DWORDLONG)1 << 39)

#define			 F_SPEECH_PART_MASK		F_NOUN|F_VERB|F_ADJECTIVE|F_ADVERB
#define         F_CASEMASK            (F_NOMINATIVE | F_GENITIVE | F_DATIVE | F_ACCUSATIVE | F_INSTRUMENTAL | F_PREPOSITIONAL)
#define         F_GENDERMASK          (F_MASCULINE | F_FEMININE | F_NEUTER)
#define         F_NUMBERMASK          (F_SINGULAR | F_PLURAL)
#define         F_SHORTMASK           (F_SHORT)
#define         F_COMPARATIVEMASK     (F_COMPARATIVE)
#define         F_LITSOMASK           (F_LITSO1 | F_LITSO2 | F_LITSO3)
#define         F_DEEPRMASK           (F_DEEPRMASK)
#define         F_INFINITIVEMASK      (F_INFINITIVE)
#define         F_TENSEMASK           (F_PAST | F_PRESENT | F_FUTURE)
#define         F_ANIMATEDMASK        (F_ANIMATED | F_INANIMATED)

#define         N_NOMINATIVE_SINGULAR        1
#define         N_PREPOSITIONAL_SINGULAR     2
#define         N_DATIVE_SINGULAR            4
#define         N_ACCUSATIVE_SINGULAR        8
#define         N_ACCUSATIVE_SINGULAR1       16
#define         N_1_SINGULAR                 32
#define         N_2_SINGULAR                 64
#define         N_NOMINATIVE_PLURAL          128
#define         N_PREPOSITIONAL_PLURAL       256
#define         N_DATIVE_PLURAL              512
#define         N_ACCUSATIVE_PLURAL          1024
#define         N_ACCUSATIVE_PLURAL1         2048
#define         N_1_PLURAL                   4096
#define         N_2_PLURAL                   4096*2
#define         N_ACCUSATIVES	N_ACCUSATIVE_SINGULAR|N_ACCUSATIVE_SINGULAR1|N_ACCUSATIVE_PLURAL|N_ACCUSATIVE_PLURAL1


#define         V_DEEPRICH_PRESENT           (1<<14)
#define         V_DEEPRICH_PAST              (1<<5)

#endif // __LANG
