/* This file is part of libmt library
   Copyright (C) 2002  Multitran Project <libmt@multitran.ru>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

// Lemmatization class functions
#ifndef __CLEMM_H     /* prevent multiple includes */
#define __CLEMM_H

#include "cbyte.h"
#include "lang.h"
#include "stem.h"
#include "cword.h"
#include "tmorf.h"

#define WRONG_WORD      -1
#define FOUNDBASE        1

// Ignored word in translation menu
#define IGNORE_ALWAYS     1
#define IGNORE_NOW        2
#define IGNORE_DELIMITER  4

#define ENGLISH_RUSSIAN_AND_DIGITS 1

extern 	int	FgetsLineWithCarry (FILE *hand, Cbyte &buff, int convert = 1);
extern int DetermineCodeTable (FILE *hand);

class Ends : public BTree {
 public:
  Ends (int langnum = 0);
  int	GetEndNumber (Cbyte &end);
  int	AppendEnd (Cbyte &end, int endnum);
  int	OpenEnds (int langnum, Cbyte &datapath = data.m_datapath);
  int	DumpEnds (Cbyte &fname, Cbyte &fname1);
  int	GetCurrentEndNumber (void);
};
Ends *GetEndsPointer (int lang, Cbyte &datapath = data.m_datapath);

#define SIMPLIFY_NEGATION								1
#define KEEP_PREPOSITIONS								2
#define DELETE_ADVERBS									3
#define LEAVE_PREPOSITIONS								4
#define ERASE_ADJECTIVES								5
#define ERASE_GENITIVE_NOUNS							6
#define ERASE_NOUNS_CONTROLLED_BY_PREPOSITIONS	7
#define SIMPLIFY_ALL SIMPLIFY_NEGATION | KEEP_PREPOSITIONS | DELETE_ADVERBS | LEAVE_PREPOSITIONS | ERASE_ADJECTIVES | ERASE_GENITIVE_NOUNS | ERASE_NOUNS_CONTROLLED_BY_PREPOSITIONS

class Lemm {
 public:
  Lemm (int lang = -1);
  Lemm (CWord *startmwp/* = NULL*/, int lang);
  Lemm (Cbyte &buff);
  ~Lemm();

  int	GetMaxWords();
  int	GetWordNumber (int wnum, int elem);
  int	GetWordStart (int pos);
  int	GetWordEnd (int pos);
  int	CanHaveWordNumber (int pos, long wordnum);
  int	Simplify (int type = SIMPLIFY_ALL);
  void	Initialize (int lang);
  int	GetDelimiterAfterWord (int pos);
  int	ExistsGermanVerb (Cbyte &buff);
  MLP	PointElement (int wnum, int elem);
  int	Exists (Cbyte &buff);
  int	LemmString (void) {Cbyte buff; return (LemmString (buff));};
  int	LemmString (Cbyte &buff, int ManyLanguages = 0);
  int	LemmString (int wpos);
  int	LemmStringWithLetterReplacement (Cbyte &buff1, byte what, byte whatto);
  int	AssignFoundValues (DWORD result, int element);

  int	NounAdjectiveMatch (int lgk1, DWORD result1, int lgk2, DWORD result2, DWORD checkbits);
  int	CheckMatchForm (Cbyte &buff, int pos1, DWORDLONG mask1, int pos2, DWORDLONG mask2, DWORDLONG checkbits);
  int	CheckMatchForm (int pos1, DWORDLONG mask1, int pos2, DWORDLONG mask2, DWORDLONG checkbits);
  int	CheckMatchForm (MWP mwp1, DWORDLONG result1, MWP mwp2, DWORDLONG result2, DWORDLONG checkbits);
  int	CheckMatchForm (MLP mlp1, DWORDLONG mask1, MLP mlp2, DWORDLONG mask2, DWORDLONG checkbits);
  int	CouldBeWordForm (MLP mlp, DWORDLONG mask, DWORDLONG result = 0);
  int	CouldBeWordForm (MWP mwp, DWORDLONG mask, DWORDLONG result = 0);
  int	CouldBeWordForm (int wordpos, DWORDLONG mask, DWORDLONG result = 0);
  int	CouldBeOnlySpeechPart (int pos, DWORDLONG);
  int	IsSameWord ();

  int	m_MaxFoundWords;
  int	m_lang;		// language to work with
  Stem	*m_StemPtr; // pointer to stem dict
  CWord *m_WordPtr;	// Word information class
  CWord m_CWord;		// local word info
  int	m_TotalSearch;
  int	m_DoStop;	// stops LemmString cycle
  int	m_InLemm;	// inside LemmString procedure
  int	m_StrictLemm;

  int	FillTMorfAnswer (struct TMorfAnswer *MainTMorfAnswerPtr);
  struct TMorfAnswer *m_TMorfAnswerPtr;
  int	m_maxTMorfAnswer;
  int	m_Period;
  int	m_YoToEReplacementPosition;
  int	m_EToYoReplacementPosition;
  int	m_WasMainWordForm;

  DWORDLONG	CompileSik_AppendForm (int spart, Cbyte &ends);
  int	Compile();
  int	m_totalcnt;
  __int16 m_siknum;
  BTree m_SikDatabase;
  int	m_MaxProcessWords;
  int	m_CheckCaseDuringLemmatization;
  int	m_StrictLemmatization;

  static int CouldBeSpeechPart (Cbyte &buff, int pos, int spart, int formnumber = 0);
  int	CouldBeSpeechPart (int wordpos, int needpart, int formnumber = 0);
  Cbyte	m_Simplified;
  int	m_ManyLanguages;
  int	m_CheckUnknownDatabase;
  Cbyte	m_UseStemPath;

 private:
  const static byte *m_seeks[MAXLANGS];;
  const static byte *m_elems[MAXLANGS];

  int	m_mainlgk;		// main lgk of word (e.g. for 襤訩 m_mainlgk = P_VERB)
  DWORD m_results;
  MWP	m_curmwp;
  int	m_InsideExists;
  int	m_MaskLgk;

  int	m_lgk;			// lgk of word (e.g. for 襤訩 m_lgk = P_ADJECTIVE)
  int	m_prich;
  int	m_OkFormCount;
  DWORD	m_DuplicateForm;
  Ends	m_Ends;
};

extern int CouldBeWordForm (Cbyte buff, int wordpos, DWORD mask, DWORD result = 0);
extern int CouldBeWordForm (Lemm CL, int wordpos, DWORD mask, DWORD result = 0);
extern int CheckMatchForm (Cbyte &one, int pos1, DWORDLONG mask1, Cbyte &two, int pos2, DWORDLONG mask2, DWORDLONG checkbits);
extern int CheckMatchForm (MWP mwp1, DWORDLONG mask1, MWP mwp2, DWORDLONG mask2, DWORDLONG checkbits);
extern int CheckMatchForm (Lemm &CL1, int pos1, DWORDLONG mask1, Lemm &CL2, int pos2, DWORDLONG mask2, DWORDLONG checkbits);
extern int CheckMatchForm (MLP mlp1, DWORDLONG mask1, MLP mlp2, DWORDLONG mask2, DWORDLONG checkbits);
extern 	int	CouldBeSameWordForms (Lemm &CL1, int pos1, Lemm &CL2, int pos2);

// Convert Result form flag information (filled by Exists)
// to word linguistic bit info
class WordForm {
 public:
  WordForm (int lang, MLP mlp);
  DWORDLONG GetAvailableForm (int lgk);
  int GetFormNumber (DWORD wordform);
  int	m_lang;
  int	m_speechpart;
  MLP	m_mlp;
};

class ProcessImport {
 public:
  int	DoWriteTranslationPair (FILE *ohand, Cbyte &original, Cbyte &translation, Cbyte &comment, Cbyte &subject);
  int	WriteTranslationPair (FILE *hand, Cbyte &original, Cbyte &trans);
  int	GetComment (Cbyte &from, Cbyte &comment);
  int	HandleComma (FILE *ohand, Cbyte &original, Cbyte &trans);

  FILE *ohand;
  Cbyte m_mainword;
};
#endif	// __CLEMM_H
