/* This file is part of libmt library
   Copyright (C) 2002  Multitran Project <libmt@multitran.ru>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

#ifndef __OHEAD_H     /* prevent multiple includes */
#define __OHEAD_H

typedef unsigned int    word;
typedef unsigned int    uint;
typedef unsigned long   DWORD;
#define MAXSUBJECTS 5  // MAX SUBJECTS ALLOWED IN TRANSLATION ENTRY, USED IN ctrabuff, cbuff
#define DOSTABLE 0
#define WINTABLE 1
#define AND_LOWER_10 4294966272  // and mask used to count each 1024th step in cycle
#define KEY_ESC           283
#define BUFFSIZE 256  // standard temporary buffer size
#define _lock _locking;
#define SH_DENYNONE _SH_DENYNO

#ifdef WIN32
#define O_RDWR _O_RDWR
#endif

#define byte char


// Globals

// Error codes 
#define ERR_NO_INDEX_FILE              -1
#define ERR_TERM_EXISTS                -2
#define ERR_NO_TRA_FILE                -3
#define ERR_NO_TERM_NUM                -4
#define ERR_NO_STEM_FILE					-5
#define ERR_USER_BREAK                 -10
#define ERR_WORD_NOT_FOUND             -11
#define ERR_INDEX_NOT_OPENED           -12
#define ERR_INVALID_LANG               -13
#define ERR_NO_HELP_ITEM               -14
#define ERR_LANG_NOT_OPENED            -15
#define ERR_BASE_NOT_OPENED            -16
#define ERR_ENTRY_TOO_LONG             -17
#define ERR_INVALID_RECORD             -18       // wrong entry passed to ADDRECORD
#define ERR_NO_SUCH_TERM               -19       // invalid term
#define ERR_CANNOT_GOTO                -20       // Cannot perform goto operation
#define ERR_INVALID_TRA_ELEMENT        -21     // invalid term element in index record
#define ERR_NO_OPEN_FILE               -10001
#define ERR_EOF								-22
#define ERR_OURBASE_INVALID				-23
#define ERR_FIELD_TOO_LONG					-24       // field too long in gettrafield
#define ERR_LOCKED                     -25       // cannot appnd because there's a record lock
#define ERR_NO_AUTOR_FILE              -26
#define ERR_NO_BASE_FILE               -27
#define ERR_FILE_TOO_LONG              -28
#define ERR_NO_DISK_SPACE              -29
#define ERR_WRONG_DB_VERSION           -30
#define ERR_NO_SUCH_FILE               -31
#define ERR_FILE_ALREADY_OPENED        1         // it's not an error
#define ERR_NOT_OUR_FILE               -33
#define ERR_NO_SIK_FILE                -34
#define ERR_NO_MEMORY                  -35
#define ERR_NO_SUCH_WORD               -36
#define ERR_INVALID_TERM_NUMBER        -37
#define ERR_NODICT                     -38
#define ERR_CANNOT_WRITE               -39
#define ERR_PHRASE_TOO_LONG            -40
#define ERR_NO_SUCH_SUBJECT_NAME       -41
#define ERR_NO_SUCH_SUBJECT_CODE       -42
#define ERR_DATABASE_NOT_OPENED        -43
#define ERR_TOO_MANY_SUBJECTS          -44
#define ERR_SUBJECT_NAME_EXISTS        -45
#define ERR_INVALID_SUBJECT_NAME       -46
#define ERR_WRONG_DATABASE_LEVEL       -47
#define ERR_CANNOT_READ                -48
#define ERR_ERROR_IN_DATABASE          -49
#define ERR_SUBJECT_NAME_PRESENT       -50
#define ERR_SHORT_SUBJECT_NAME_PRESENT -51
#define ERR_SUBJECT_NAME_NOT_SPECIFIED -52
#define ERR_SUBJECT_NAME_IN_DIFFERENT_LANGUAGE -53
#define ERR_EMPTY_FIELD                -54
#define ERR_NOWORDINI                  -55
#define ERR_USER_INTERRUPT             -56
#define ERR_TOO_MANY_TRANSLATIONS		-57
#define ERR_TOO_MANY_USERS             -58
#define ERR_NO_NETLOGFILE					-59
#define ERR_EXPIRED_TIME					-60
#define ERR_VOID_INSTANCE					-61 // call to non-defined class member
#define ERR_SUBJECT_CODE_PRESENT       -62
#define ERR_RECORD_ALREADY_EXISTS		-63
#define ERR_WRONG_LANGUAGE					-64
#define ERR_PERMISSION_DENIED				-65
#define ERR_CANNOT_DETERMINE_LANGUAGE	-66
#define ERR_UNKNOWN_SHORT_SUBJECT_NAME -67
#define ERR_NO_SHORT_SUBJECT_NAME		-68
#define ERR_NO_SUGGEST						-69
#define ERR_NO_SUCH_SIK						-70
#define ERR_FINISH							-71
#define ERR_INVALIDE_LEVEL_SIZE			-72
#define ERR_ENTRY_ALREADY_EXISTS			-73
#define ERR_INVALID_DATABASE_NAME		-74
#define ERR_NO_SUCH_ELEMENT				-75
#define ERR_INVALID_LANGUAGE				-76
#define ERR_UNKNOWN_FULL_SUBJECT_NAME  -77
#define ERR_DUPLICATE_MESSAGE				-78
#define ERR_FILE_ALREADY_EXISTS			-79
#define ERR_CANNOT_CREATE_FILE			-80
#define ERR_CONVERSION_TO_HTML			-81
#define ERR_NULL_STRING						-82
#define ERR_ENTRY_INCLUDED_IN_OTHER_ENTRY	-83
#define ERR_CANNOT_OPEN_SUBJECT_TXT		-84
#define ERR_FILE_ALREADY_PROCESSED		-85
#define ERR_EMPTY_USERNAME					-86
#define ERR_TOO_MANY_PASSWORD_ATTEMPTS -87
#define ERR_CANNOT_SAVE_FORUM_MESSAGE  -88

#define NO_STNFILE -1
#define NO_BASFILE -2
#define NO_EXCFILE -3
#define NO_TRAFILE -4
#define NO_INDFILE -5
#define NO_AUTFILE -6
#define NO_MSGFILE -7

// Interpreting SIK file
#define CHANGELEM 25
#define UMLAUT_STUFF       175 // 240
#define GO_PARTICIPLE      176 // 241
#define NO_FORM            177 // 242
#define IMMEDIATE_END      178 // 243
#define ALTERNATIVE_END    179 // 245
#define DOUBLE_LETTER      180 // 246
#define SET_BLOCK          181 // 247
#define BLOCK_TYPE         182 // 248
#define GLOBAL_SUFFIX      183 // 249
#define SKIP_SUFFIX        184 // 250
#define DIRECT_SUFFIX      185 // 251
#define END_CONT_SUFFIX    186 // 252
#define CONT_SUFFIX        187 // 253
#define SET_LGK            188 // 254
#define REFERENCE_SUFFIX   189 // 255

#define GET_FORMS  1
#define CHECK_WORD 2
#define BASE_EXT  "BAS"
#define MESS_EXT  "MSG"
#define AUT_EXT   "AUT"
#define PREFIX_EXT  "PRE"
#define EXT_TRANSCRIPTION   "TRC"   // transcription database
#define EXT_CARRY           "CAR"   // carry database

#define MAX_PHRASE   50    // max words in phrase
#define NEW_MAX_PHRASE_SIZE 50

#endif
