/* This file is part of libmt library
   Copyright (C) 2002  Multitran Project <libmt@multitran.ru>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

// Stem dictionary class
#ifndef __STEM_H     /* prevent multiple includes */
#define __STEM_H

#define NB_NOTPRESENT   1   /* NEWBASE                                        */
#define NB_EXPANDS      2   /* New info expands meaning of previously existed */
#define NB_COINCIDE     3   /* absolutely the same                            */
#define NB_INCLUDED     4   /* new info is contained in the older one         */

/* Working with DataBase files */

/* newbase - adding bases */
#define NB_CHECKBASE        1
#define NB_ADDBASE          2
#define NB_DELETEBASE       4
#define NB_ASSIGNWORDNUMBER 8   // reassign word number 
#define NB_NOEXTEND         16  // this word is appended as totally new without merging with possibly exisiting words
#define CHECK_ONLY_PART_OF_SPEECH 1 // used by NewStem()

#define NO_AUTODUPLICATE	0
#define AUTODUPLICATE		1

#include "btree.h"
#include "getbit.h"
#include "cword.h"
#include "lang.h"

#define	STEM_INFO_BLOCK_SIZE 7

class Stem : public BTree {
public:
  //   Stem	(int langnum = 0);
  Stem();
   ~Stem();
	long	CountWords ();
	void	RecalculateInfoBlocks ();
   int	Seek (byte *what, int size);
	int	Seek (Cbyte &buff, int size = -1) {return Seek (buff.b, size);};
	int	SeekRecordByMlp (Cbyte &buff, MLP mlp);
   int	CloseDb (void);
   int	OpenStem (int langnum, Cbyte datapath = (Cbyte)"");
   int	SetLgk (int num, int lgk);
   int	GetLgk (int num);
   int	SetSik (int num, int sik);
   int	GetSik (int num);
   int	SetWordNumber (int i, long wordnum);
   long	GetWordNumber (int i);
   int	SetBlock7 (int num, byte *lblock7);
   int	GetBlock7 (int num, byte *lblock7);
   int	GetMaxBlocks (void);
   byte *PointBlock7 (void);
   byte *GetOriginal (void);
   int	DeleteBlock7 (int num);
   int	AddBlock7 (long wordnum, int sik, int lgk);
   int	GetStemCase ();
   void	SetStemCase (int ourcase);
   int	GetRecordCase (void);
   int	CompareCase (Cbyte &buff);
   int	NewStem (Cbyte &wword, int size, long *wordnum, int newlgk, int newsik, int ActionType);
	int	NewStem (Cbyte &wword, int size, int newlgk, int newsik, int type);
   int	AppendStem (byte *lbase, int size, byte *pass, int psize, int ccase);
   int	ApplyStemCase (Cbyte &buff);
   int	AppendFromStemDict (Stem *DbSource);
   void	DumpStems (int type, FILE *ohand);
   int	GetTextStatistics (Cbyte &fname);
   int	MakeCopy (long maxnum);
	int	CopyReferencedWordNumbers (OperateBit *bitptr, Cbyte &path);
	int	AutoAppend (Cbyte &buff, int needspart = -1, int doappend = 1);
	int	SynthesizeWordForms (int pos, Cbyte &buff, int type = SYNTHESIZE_ALL_FORMS);
	int	FindWordNumbersBiggerThanAllowed ();
	long	FindAndSetBiggestWordNumber (int type = 0);
	int	DeleteWordNumbersBiggerThanSpecified (long max);
	int	GetStem (Cbyte &buff);
	int	GetStemSize (void);
	int	GetOffsetOfFirstInfoBlock (void);
	long	CollectAccessibleWordNumbers();
	static const void	ReOpenDatabases (int justclose = 0);

	int DumpContents(char *fname);

	int	m_MaintainAdditionalIndexes;
	int	m_strict;
   long	m_maxwords;
	int	m_InNewStem;
	int	m_CheckType;
	int	m_AutoDuplicate;


	mylinfo m_MyLinfo; // used in AutoAppend function

	OperateBit m_BitArray;

private:
   int	m_DefaultLanguage; // same as * in Stems[*] structure, used for default stem opening
};

class BackWords : public BTree
{
public:
	BackWords (int lang);
	int	OpenBackWords (int langs, int create = 0);
	int	AppendBackWord (Cbyte &word, int lgk);
	int	DeleteBackWord (Cbyte &word, int lgk);
	int	GetRealBackWord (Cbyte &wword);
	int	SeekWord (Cbyte &buff);
	int	Rebuild();
};

class Words : public BTree
{
public:
	Words (int lang);
	int	OpenWords (int langs, int create = 0);
	int	AppendWord (Cbyte &word, int lgk);
	int	DeleteWord (Cbyte &word, int lgk);
	int	GetRealWord (Cbyte &wword);
	int	Rebuild();
};

class FindLetters : public BTree
{
public:
	FindLetters (int lang);
	int	OpenFindLetters (int lang, int create = 0);
	int	GenerateFindLetters();
	int	CollectWords (Cbyte &wword, Cbyte &out);
	int	Export();
	void	SortWord (Cbyte &buff);
	int	DeleteWord (Cbyte &wword);
	int	AppendWord (Cbyte &wword, int lgk);
	int	StoreAppendWord (Cbyte &wword, int lgk, int del = 0);
	int	ImportAppendWord ();

	int	m_FindNouns;
	int	m_FindVerbs;
	int	m_FindAdjectives;
	int	m_FindOther;
	int	m_MatchCase;
	int	m_AnyOrder;
	int	m_SortBySize;
	int	m_RussianYoEqualsE;
	int	m_OnlyGivenLetters;
};

class NumWord : public BTree
{
public:
	NumWord (int lang);
	int	AppendWord (long wordnum, Cbyte &buff);
	int	DeletedWord (long wordnum, Cbyte &buff);
	int	GetWord (long wordnum, Cbyte &buff);
	int	Rebuild();
};

class WordFreq : public BTree
{
public:
	WordFreq (int lang);
	long	GetWordFrequency (long wordnum);
	long	GetWordNumber ();
};

class PrefWord : public BTree
{
public:
	PrefWord (int lang);
	int	OpenPrefWord (int lang, int type = 0);
	int	AppendPrefWord (Cbyte &wword, Cbyte &pref);
	int	FindPrefixesForWord (Cbyte &wword, Cbyte &getpref);
	int	ExportPrefWordToText (Cbyte &fname);
	int	ImportPrefWordFromText (Cbyte &fname);
	int	GetRealWord (Cbyte &rword);
	int	Rebuild ();
};
class WordsForPrefix : public BTree
{
public:
	WordsForPrefix (int lang);
	int	OpenWordsForPrefix (int lang, int type = 0);
	int	FindWordsForPrefix (Cbyte &wword, Cbyte &getpref);
	int	AppendWordsForPrefix (Cbyte &wword, Cbyte &pref);
	int	Rebuild();
	int	ExportWordsForPrefixToText (Cbyte &fname);
};

extern Stem *GetStemPointer (int lang, Cbyte &datapath = data.m_datapath);

class UnknownDb : public BTree
{
public:
	int GetWordNumber (Cbyte &buff);
	int	AppendNonFound (Cbyte &buff);
};

class NounGenderDb : public BTree
{
public:
	NounGenderDb (int lang);
	int	AddNoun (Cbyte &buff, int gender);
	int	GetNounGender (Cbyte &noun);
	int	GenereateDb();
	int	Export (Cbyte &fname);

	int	m_lang;
};

class FullStem : public BTree
{
public:
	FullStem (int lang);
	int	AddStem (Cbyte &buff, long wordnum, long result, __int16 lgk);
	int	SeekStem (Cbyte &buff);
	int	Generate();
};



#endif // __CSTEM
