/* This file is part of libmt library
   Copyright (C) 2002  Multitran Project <libmt@multitran.ru>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

// Subject class

#ifndef __SUBJECT_H     /* prevent multiple includes */
#define __SUBJECT_H

#include "ohead.h"
#include "dbpair.h"
#include "lang.h"

#define GLOBALSUBJECT "globalsubject" // for import
#define MASK_INACTIVE_SUBJECT_FLAG	0x7FFFFFFF
#define INACTIVE_SUBJECT_FLAG			0x80000000
#define MAX_ALLOWED_SUBJECTS 2048 // supposedly maximum number of supportes subjects

class SubjectStat
{
public:
	SubjectStat (int langin, int langout);
	~SubjectStat();
	int	GenerateSubjectStat();
	long	GetSubjectCount (int snum);
	int	SetSubjectCount (int snum, long max);
	int	m_hand;
	Cbyte	m_fname;
	int	m_langs[2];
};

class SubjectOrder
{
public:
	SubjectOrder();
	~SubjectOrder();
	int	ReadOrder();
	int	WriteOrder();
	int	GetSubjectOrder (int subjectnum);
	void	SetSubjectOrder (int subjectnum, int position);
	int	GetSubjectForPosition (int pos);
	int	GetSubjectActivity (int subjectnum);
	void	SetSubjectActivity (int subjectnum, int type);
	void	MoveSubjectAbove (int fromsub, int tosub);

   int	m_MaxSubjects;
	int	m_WasActivityChange;
	int	*m_Sorder;
	byte	*m_Sactivity;
};

#define MAX_ALLOW_SUBJECTS 1024
#define LEAVE_OPEN 1
#define GET_SUBJECT_DESCRIPTION	1
#define GET_FULL_SUBJECT_NAME		2

class CSubject : public SubjectOrder
{
public:
   CSubject (int lang = 0);
	~CSubject ();

	int	IsMarkedSubject (int snum);
	int	GetSourceDescription (int snum, Cbyte &buff, int type);
	void	FillShortSubjects (int lang);
	int	OpenSubject (byte *mode = "r", int leaveopen = 0);
	int	AppendSubject (Cbyte &fullsting, byte *delim);
	int	m_WasActivityChange;
   int	GetSubjectName (int code, Cbyte &fullname, int lang);
   int	GetMaxSubjects ();
   int	DeleteSubject (int scode);
   void	AssignLanguage (int lang);
   int	IsShortSubjectNamePresent (int lang, Cbyte &shortname, int code);
   int	AppendSubject (Cbyte &fullrussian, Cbyte &shortrussian, Cbyte &fullenglish, Cbyte &shortenglish, int code = -1);
   int	EditSubject (int code, Cbyte &fullrussian, Cbyte &shortrussian, Cbyte &fullenglish, Cbyte &shortenglish);
	int	GetSubjectNumberFromOrder (int pos);
	int	GetSubjectOrderFromOrder (int subjectnum);
   int	GetSubjectCode (Cbyte subjectname, int lang);
   int	GetShortSubjectCode (Cbyte subjectname, int lang);
   int	GetShortSubjectName (int code, Cbyte &shortname, int lang);
   int	GetFullNameFromShort (Cbyte &shortname, Cbyte &fullname);
   int	GetShortNameFromFull (Cbyte &fullname, Cbyte &shortname);
	int	GetNextAlphabeticSubjectName (int code, int dir = 1);
	static const int GetGlobalSubject (Cbyte &buff);
	static const int FormGlobalSubjectLine (int GlobalSubject, Cbyte &buff);
	void	CollectAlphabeticSubjectNames (Cbyte &buff, int lang, int usenumbers = 1);
   int	m_lang;
   int	m_backlang;
//	DbPair m_Subj[MAXLANGS];
//	DbPair m_SSubj[MAXLANGS];
	Cbyte	m_ExistingName;
	FILE *m_shand;
	Cbyte m_ShortNames[2][MAX_ALLOW_SUBJECTS];
	Cbyte m_FullNames[2][MAX_ALLOW_SUBJECTS];
	Cbyte m_SourceDescription[MAX_ALLOW_SUBJECTS];
	byte  m_MarkSubject[MAX_ALLOW_SUBJECTS];
	int	m_MaxSubjects;

private:
   int	m_OutputCodeTable;
};

CSubject*	GetSubjectPointer();
#define G_Subject (*GetSubjectPointer())

#endif  // __SUBJECT_H	 
