/* This file is part of libmt library
   Copyright (C) 2002  Multitran Project <libmt@multitran.ru>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

#ifndef __TPROCESS_H__
#define __TPROCESS_H__
#include "cword.h"
#include "lemm.h"
#include "typein.h"
#include "noclass.h"
#include "index.h"
#include "transcri.h"
#include "abcsubj.h"

#define MAXSCREENSTRINGS 1000 // max lines in screen output
#define OFFSET_TRANSLATION		10
#define OFFSET_COMMENT			30
#define MAXINDEXSTRINGS 10		// number of index strings that could be viewed at once
#define START_LINE_OFFSET 10	// free space at the start of each line
#define TOP_OFFSET 50			// offset from top of screen to adjust for toolbars line
#define MAXTB 300					// max translation entries

// Enable/disable buttons in toolbar
struct ShowArrayStruct
{
	int ShowViewPhrases;
	int ShowSplitPhrase;
	int ShowShowAll;
	int ShowShowAllButton;
	int ShowCopyTranslations;
	int ShowTranslateBack;
	int ShowSkipTranslation;
	int ShowMoveToPrevious;
	int ShowMoveToNext;
	int ShowViewSynonims;
};

#define MAX_INCLUSION_SIZE 2500
class TranslationProcess
{
public:
	TranslationProcess (int langin = 0, int langout = 0);
	~TranslationProcess();

	int	SortTermsByOriginal();
	int	CollectFoundWord (int pos, Cbyte &DisplayPhrase);
	void	Initialize (int langin, int langout);
	int	GetMaxProcessedBlocks();
	int	HasTranslations (int elem);
	int	CollectTermNumbersForMlp (MWP mwp, int elem, long RecommendTermNumber);
	int	m_SomethingFound;
	void  SaveCurrentTranslation();
	int	ProcessSentence();
	int	ProcessPhrase1();
	int	GetCurrentOriginalWord (int num, Cbyte &buff);

	int	GatherTermNumbers (long needtermnumber = -1);
	int   ProcessBuffer (Cbyte &curbuff1, int type = 1, int startword = 0);
	int	DoProcessBuffer (Cbyte &curbuff1, int type = 1, int startword = 0);

	Cbyte m_ReallyProcessedWord;
	int	m_ActiveTBnum; // current active translation buffer number
	int	m_CurTranslation;  // current translation
	int	m_StartArrayLine;
	Index	*m_Iptr;   // Pointer to currently used index direction
	int	m_MaxTB; // max translation blocks
	TraBuff *m_TB; // [MAXTB]; // translation buffers to store translation entries for current word on the screen 
	int	m_langs[2]; // working language pair
	int	m_ReplaceWholeText;
	int	m_InProcess;	// = 1 if we're inside ProcessBuffer
	int	m_AllReady; // =1 when it's ok to translate on focus
	int	m_MaxNoPhrase; // // clear nophrase struct - translate from scratch
	int	m_FirstShow;
	int	m_MoreTranslationsToShow;
	NoPhrase m_NoPhrase[MAXNOPHRASE];
	int	m_CheckSubjects;
	int	m_WholeRequestTextFound;
	int	m_InsertNonfoundWordInOutput;

	Lemm	m_Lemm;
	int	m_DoSentence;
	CWord	m_Output;  // structure of translated words
	Transcription *m_Tptr;
	int	m_ForceLanguages;
	byte *m_SaveInclusions;
	byte *m_Inclusions;
	int	m_ShowProgress;
};

extern int CheckEntryExistence (TraBuff &TB, int *same);

#endif//__TPROCESS_H__
