/* This file is part of libmt library
   Copyright (C) 2002  Multitran Project <libmt@multitran.ru>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

// Translation buffer class functions

#ifndef __CTRABUFF_H     /* prevent multiple includes */
#define __CTRABUFF_H

#include "dict.h"
#include "cword.h"
//#include "tprocess.h"

#define USER_FIELD_OPENING_BRACKETS "<<<"
#define USER_FIELD_CLOSING_BRACKETS ">>>"
#define PICTURE_FIELD_OPENING_BRACKETS "<<<<"
#define PICTURE_FIELD_CLOSING_BRACKETS ">>>>"

// Translation entry field values
#define         F_NOTE          6
#define         F_EXAMPLE       7
#define         F_USER			  8
#define         F_SOURCE        9
// Do not use code              10 !!!
#define         F_TIME			  11
#define         F_MAXTEXTFIELD  11
#define         F_PICTURE		  12
// Do not use code              13 !!!
#define         F_EMAIL			  14
#define         F_SUBJECTFIELD  15
#define         F_DUMP          17
#define         F_COLOR         18
#define         F_BULLET        19
#define         F_BULLET1       20

// Returns of TraGet function
#define TRAGET_INACTIVE_SUBJECT  2
#define TRAGET_OK						1
#define TRAGET_INVALID_TERM      0

#define USE_TERM_NUMBERS			1  // for ConvertTrabuffToString
#define USE_ARROW						1

// These definitions should be in one group !
#define ONE_DIRECTION			64
#define BOTH_DIRECTIONS			128
#define JUST_WRITE_TO_SCRIPT  512
#define JUST_CHECK_EXISTENCE	1024
#define CHECK_UNKNOWN_WORDS_IN_BOTH_DIRECTIONS	2048
#define NEW_ENTRY_REPLACES_EXISTING_ENTRY			2048*2

// Import: local subjects treatment
#define LOCAL_SUBJECT_REPLACES_MAIN 1   
#define LOCAL_SUBJECT_BECOMES_SECOND 2   
#define LOCAL_SUBJECT_IS_IGNORED 3   

#define IGNORE_INTERNAL_BRACKETS	0
#define SHOW_INTERNAL_BRACKETS	1

#define EXTRACT_ALL_BRACKETS		0
#define EXTRACT_FINAL_BRACKETS	1
#define BOTH_ARROWS	1
#define RIGHT_ARROW	2
#define LEFT_ARROW	4

#define BULLET_DASH		1
#define BULLET_NUMERIC	2

#define MOVE_TRAILING_ORIGINAL_BRACKETS_TO_COMMENT		1
#define MOVE_TRAILING_TRANSLATION_BRACKETS_TO_COMMENT	2

#define IGNORE_CASE					1   // in CheckSameFields
#define IGNORE_NOLETTER_SYMBOLS	2
#define IGNORE_YO						4
#define IGNORE_BRACKETS				8

#define MAX_TRABUFF_SIZE 1500 // max trabuff written to dict, otherwise to separate text file

class TraBuff {
public:
   TraBuff (int langin, int langout);
	TraBuff (Cbyte &buff);
   TraBuff ();
   ~TraBuff();
	void	Construct (int langin, int langout);

	int	GetOriginal (Cbyte &buff);
	int	SetTranslation (Cbyte &buff);
	int	SetOriginal (Cbyte &buff);
	int	GetTranslation (Cbyte &buff);

	int	CheckEntryFields ();
	void	FreeObject();
	int	AddToComment (Cbyte &buff);
	int	GetCommentsFromField (int fnum, Cbyte &bbb);
	int	DumpEntry (long termnum, byte *fname);
	void	CollectShortSubjectNamesToString (Cbyte &buff);
	int	FormDumpWordString (int direction, Cbyte &where);
	int	CompareEntries (TraBuff &TB1, TraBuff &TB2, int type = IGNORE_CASE|IGNORE_YO);
   int	SettleWords();
   int	SettleFirstWords (int spart1, int spart2);
   int	ProcessEntry (int maxdirec = BOTH_DIRECTIONS); // process either in one or 2 directions
   int	FillFirstNumbers (int autoappend = 0);

	void	AdjustYo();
	int	StripInternalBrackets (Cbyte &buff);
	int	GetComment (Cbyte &buff);
	int	GetMaxSubjects ();

   int	operator == (TraBuff &TB);
   void	operator = (TraBuff *TB);
   void	operator = (TraBuff &TB);

	int	GetBufferSize();
	int	GetField (int num);
	int	GetField (int num, Cbyte &buff, int showall = SHOW_INTERNAL_BRACKETS);
   int   SetField (int num, Cbyte &buff); 
   int   GetFieldSize (int num);
   int   CheckOtherFields (void);  // check if additional fields are present
   int   DeleteField (int num);
	int	NewDeleteEntry (int type =	BOTH_DIRECTIONS);
   int   CheckFields (void);
   int   GetSubject (int number = 0);    // get current entry's subject from m_subjects[]
   int   GetSubjectFromBuffer (int number = 0);    // get current entry's subject from buffer
   int   IsEmpty();
   int   ReadEntry (long termnum = 0);
   int   ReadEntry (byte *terms);
   int   ToggleBullet (void);
   int   GetBullet (int langin = 0, int langout = 0);
   int   SetBullet (int val);
   int   LocateFirstEntry (Cbyte &searchstring, int bitflags); // search tradict for substring from top of file
   int   LocateNextEntry (); // search tradict for next substring starting from current record
   int   LocateCycle ();
   int   InitTraBuff (int langin, int langout);
   static const int CompareField (TraBuff &tb1, TraBuff &tb2, int fieldnum, int parameter = 0);
	static const int ExtractHttpReference (Cbyte &comment, Cbyte &http);
	int	CreateLongCommentFileName (Cbyte &path);
	int	ExpandComment(Cbyte &buff);
	int	MoveBracketsToComment();

	Cbyte	m_CommentFileName;
	int	m_DoTrim;
	int	m_UseSubjects;
	int   IsActiveSubject (void);   // checks if current entry belongs to active subjects
	static const int   IsSameSubject (TraBuff &TB1, TraBuff &TB2);    // checks if current entry belongs to active subjects
	int   CollectSubjectNames (Cbyte &buff);
	int	ReadLine();
	int	WriteLine (Cbyte fname = (Cbyte)"");
	int	WriteLine (FILE *hand);
	int   OpenFile (Cbyte &fname);
	int	ConvertStringToTrabuff (Cbyte &buff, long termnum = 0);
	int	ConvertTrabuffToString (Cbyte &retbuff, int allelements = 1, int UseTermNumber = 0);
	int ConvertToUTF(Cbyte &buff);


   void  SetSubject (int subj, int number = 0); // set subject to m_subjects[]
   void  NewTraBuff ();    // clear trabuff in memory   
	int	AssignBuffer (long size, int doclear = 1);
	int AssignBuffer (Cbyte &buff);
	void  NewTra ();        // clear trabuff in memory + clear m_termnum
   void  CollectFlagData();
   void  AppendFlagData();
	int	FillSubjects (Cbyte &buff);
	void  GetDirection (Cbyte &buff);
   int	CollectEntry (); // read entry starting with current record
   long  WriteEntry (void);
   byte *GetFieldStart (int num);
	void	ReverseLanguage();
	void	ClearSubjects ();

	static void ExtractOrigTrans (Cbyte &from, Cbyte &orig, Cbyte &trans, int langin, int langout);
	static void ExtractOrigTransComment (Cbyte &buff, Cbyte &orig, Cbyte &trans, Cbyte &comment, int lang1, int lang2);
	static void	ExtractSlashes (Cbyte &from, Cbyte &where);
	static byte *LocateTranslationStart (Cbyte &buff, int lang, int lang1);

   Cbyte m_trabuff;        // translation buffer
   long	m_termnum;
	long	m_termnumWithoutComment;
	long	m_SupposedTermNumber;
	static int	m_DirectionType;
   int	m_langs[2];
	int	m_CodeTable;
	int	m_MaxSubjects;
   byte	m_curnums[2][(NEW_MAX_PHRASE_SIZE + 1)*3];
   int	m_curmax[2];
	int	m_AutoAppend;
   int	m_AutoMaxPhrase[2];			// longest phrase for automatic appending
   int	m_AutoBack;					// perform automatic backward processing of the entry
   int	m_keep_buffer_size;  // = 1 - do not reallocate buffer size
   int	m_SpeechPart;       // speech part for screen display
//	int	m_lgk;
//	int	m_IndexRecordNumber;
	int	m_IndexEntryPosition;
	int	m_SubjectActivity; // filled by TraGet. = 1 if entry subject is active
	int	m_AllowDoubleSubjects;  // local subjects replace global in import
	int	m_GlobalSubject;			// global subject (for import)
	int	m_UnknownSubjectName;   // set to 1 by ReadLine if unknown subject name encountered
	long	m_IndexOffset;
	FILE *m_ihand;
	FILE *m_ohand;
	Cbyte m_TextPath;					// to write output text files in import mode
   Dict *m_DictPtr;
	Cbyte m_datapath;
   __int16 m_subjects [MAXSUBJECTS];  // subject codes
   __int16 m_source;       // source code
   __int16 m_bullet[2];    // bullets for both directions
	int	m_RecommendLgk;	// used for import of combinatory dict

	int	m_LocalSubjectBehavior; // import processing of local subjects
	int	m_IgnoreCase;					// ignore case during import
	int	m_IgnoreNoLetterSymbols;	// ignore noletter symbols during import
	int	m_MoveTrailingBracketsToComment;

//	MLP	m_mlp; // test for displaying English exceptions
	struct mylinfo m_Mylinfo;
	int	m_IndexRecordNumber;
   CWord	m_cword[2];			      // starting offsets to word structures
	int	m_HiddenDialog;

	// used for Internet appending
	Cbyte m_Message;
	int	m_dir;

private:
   Cbyte m_SearchString;
   int m_searchflags;
};

extern void	ExchangeTraBuff (TraBuff &TB1, TraBuff &TB2);

#define	IMPORT_HAS_UNKNOWN_WORDS                  "unknentr.123"
#define	IMPORT_NEW_STEM                           "unknstem.123"
//#define IMPORT_NEW_FIRST_STEM                     "new1word.123"
//#define IMPORT_NEW_OTHER_STEMS                    "newword.123"
#define	IMPORT_TOTAL_EXISTS                       "exists.123"
#define	IMPORT_ENTRY_HAS_OTHER_TRANSLATIONS       "othertra.123"
#define	IMPORT_ENTRY_ABSENT_IN_FORWARD_DIRECITON  "dir1new.123"
#define	IMPORT_ENTRY_ABSENT_IN_BACKWARD_DIRECITON "dir2new.123"
#define	IMPORT_DIFFERENT_COMMENT                  "diffcomm.123"
#define	IMPORT_DIFFERENT_SUBJECT                  "diffsubj.123"
#define	IMPORT_DIFFERENT_SUBJECT_AND_COMMENT      "diffsuco.123"
#define	IMPORT_ERROR_APPEND								"error.123"
#define	IMPORT_UNKNOWN_SUBJECT							"unknsubj.123"

// Search flags
#define SEARCH_ANY_CASE             1
#define SEARCH_ORIGINAL_FIELD       2
#define SEARCH_TRANSLATION_FIELD    4
#define SEARCH_ADDITIONAL_FIELDS    8
#define SEARCH_INSIDE_WORDS         16
#define SEARCH_CURRENT_DIRECTION    32

#define MAXBULLETS 1            // bullet toggles between 0 and 1
#endif  // __CTRABUFF
